<?php

require_once("$rootdir/classes/DBConnection.php");

class Project extends DB_row {

	function &getCache() {
		static $cache;
		if(is_null($cache)) {
			$conn = &DBConnection();
			$conn->setFetchMode(DB_FETCHMODE_OBJECT, 'Project');
			$root->childIDs = array();
			$cache = array(0=>&$root);
			$res = $conn->query('select * from Project');
			while($project = &$res->fetchRow()) {
				$cache[$project->Project_ID] = &$project;
			}
			$conn->disconnect();
		}
		return $cache;
	}

	function &get($Project_ID=null) {
		if(is_null($Project_ID)) {
			return new Project;
		}
		$cache = &Project::getCache();
		return $cache[$Project_ID];
	}

	function &getCustomerIDs() {
		if(! isset($this->customerIDs)) {
			$conn = &DBConnection();
			$this->customerIDs = &$conn->getCol('select Customer_ID from Customer_Assignment where Project_ID=?', 0, array($this->Project_ID));
			$conn->disconnect();
		}
		return $this->customerIDs;
	}

	function &getCustomers() {
		global $rootdir;
		require_once("$rootdir/classes/Customer.class.php");
		$customers = array();
		foreach($this->getCustomerIDs() as $customerID) {
			$customers[] = &Customer::get($customerID);
		}
		return $customers;
	}
	
	function &getRepresentativeIDs() {
		if(! isset($this->representativeIDs)) {
			$conn = &DBConnection();
			$this->representativeIDs = &$conn->getCol('select RA.Representative_ID from Representative_Assignment RA left join Customer_Assignment CA on RA.C_Assignment_ID = CA.C_Assignment_ID and CA.Project_ID=?', 0, array($this->Project_ID));
			$conn->disconnect();
		}
		return $this->representativeIDs;
	}
		
	function &getRepresentatives() {
		global $rootdir;
		require_once("$rootdir/classes/Representative.class.php");
		$representatives = array();
		foreach($this->getRepresentativeIDs() as $representativeID) {
			$representatives[] = &Representative::get($representativeID);
		}
		return $representatives;
	}
	
	function getUserID($ProjectRole_ID) {
		$conn = &DBConnection();
		$userID = &$conn->getOne('select User_ID from Project_Assignment where Project_ID=? and ProjectRole_ID=?', array($this->Project_ID, $ProjectRole_ID));
		$conn->disconnect();
		return $userID;
	}
	
	function &getUser($ProjectRole_ID) {
		global $rootdir;
		require_once("$rootdir/classes/User.class.php");
		return User::get($this->getUserID($ProjectRole_ID));
	}		
	
	function &getPM() {
		if(! isset($this->pm)) {
			$this->pm = &$this->getUser(1);
		}
		return $this->pm;
	}
	
	function &getQM() {
		if(! isset($this->qm)) {
			$this->qm = &$this->getUser(2);
		}
		return $this->qm;
	}	

	function setHierarchy() {
		static $flag = False;
		if($flag) {
			return;
		}
		$cache = &Project::getCache();
		function processProject(&$project, &$Project_ID, &$cache) {
			if(! isset($project->childIDs)) {
				$project->childIDs = array();
			}
			$parent = &$cache[$project->HigherProject_ID];
			if($project->HigherProject_ID) {
				$project->parent = &$parent;
			}
			$parent->childIDs[] = $Project_ID;
		}
		array_walk($cache, 'processProject', $cache);
		$flag = True;
	}

	function addSubprojectIDs(&$IDs) {
		$IDs[] = $this->Project_ID;
		foreach($this->childIDs as $Project_ID) {
			$project = &Project::get($Project_ID);
			$project->addSubprojectIDs($IDs);
		}
	}

	function &getSubprojectIDs() {
		$this->setHierarchy();
		if(! isset($this->subprojectIDs)) {
			$this->subprojectIDs = array();
			$this->addSubprojectIDs($this->subprojectIDs);
		}
		return $this->subprojectIDs;
	}

	function addAll(&$projectArray, $Project_ID, $sortField) {
		if($Project_ID) {
			$project = &Project::get($Project_ID);
			$projectArray[] = &$project;
		}
		else {
			$project = &Project::get(0);
		}
		$childFields = array();
		foreach($project->childIDs as $ID) {
			$child = &Project::get($ID);
			$childFields[$ID] = $child->$sortField;
		}
		asort($childFields);
		foreach($childFields as $ID=>$value) {
			Project::addAll($projectArray, $ID, $sortField);
		}
	}

	function &getAll($Project_ID=null, $sortField='ProjectName') {
		Project::setHierarchy();
		$projectArray = array();
		Project::addAll($projectArray, $Project_ID, $sortField);
		return $projectArray;
	}
	
	function &getUserEffort() {
		if(! isset($this->userEffort)) {
			global $rootdir;
			require_once("$rootdir/classes/User.class.php");
			$conn = &DBConnection();
			$projectIDList = implode(',', $this->getSubprojectIDs());
			$rows =&$conn->getAll(
				'select PA.User_ID, sum(time_to_sec(P.Duration)) DurationSec, P.InChargeAble '
				.'from Performance P left join Project_Assignment PA '
				.'on P.P_Assignment_ID=PA.P_Assignment_ID '
				."where PA.Project_ID in ($projectIDList) and P.Status='A' "
				.'group by PA.User_ID, P.InChargeAble',
				DB_FETCHMODE_OBJECT);
			$conn->disconnect();
			$this->userEffort = array();
			$this->effortSec = array();
			foreach($rows as $row) {
				$userEffort = &$this->userEffort[$row->User_ID];
				$userEffort->user = User::get($row->User_ID);
				$userEffort->effortSec[$row->InChargeAble] += $row->DurationSec;
				$this->effortSec[$row->InChargeAble] += $row->DurationSec; 
			}
		}
		return $this->userEffort;
	}
	
	function getEffortSec($InChargeAble=null) {
		$this->getUserEffort();
		return is_null($InChargeAble) ? array_sum($this->effortSec) : $this->effortSec[$InChargeAble];
	}
	
	function getCalculatedProgress() {
		if($this->TargetWorkingTime > 0) {
			return round(100*$this->getEffortSec()/3600/$this->TargetWorkingTime);
		}
	}

	function &getLink($table) {
		global $rootdir;
		require_once("$rootdir/classes/Cache.class.php");
		$thisArray = (array)$this;
		return Cache::get($table, $thisArray[$table.'_ID']);
	}

	function checkViewDetailsRight(&$user) {
		return $user->isAdmin() || $this->isPM($user);
	}
	
	function checkEditDetailsRight(&$user) {
		return $user->isAdmin() || $this->isPM($user);
	}
	
	function checkDeleteRight(&$user) {
		return $user->isAdmin();
	}

	function isPM(&$user) {
		$this->setHierarchy();
		if($user->hasProjectRole(1, $this)) {
			return true;
		}
		if(is_null($this->parent)) {
			return false;
		}
		return $this->parent->isPM($user);
	}

	function isActive() {
		return $this->Status_ID <> 5;
	}

	function getPath() {
		$this->setHierarchy();
		return isset($this->parent) ? $this->parent->getPath().'/'.$this->ProjectName : $this->ProjectName;
	}

}

?>