<?php
/**
* Benjamin Kuz <ben@slax0rnet.com>
* http://ben.nullcreations.net
*
* sortObjArray is a class to sort an array of objects by one or two property name(s).
* If two property names are given, the first one is treated as a group, similar to mysql's
* GROUP BY clause.
**/
class sortObjArray {

    var $property;
    var $objArr;
    var $direction;
    var $orderBy;

    /**
     * void sortObjArray( array $objArr, string $orderBy [, int $d1 ,string $sortBy, int $d2]) :: constructor (public)
     *
     * This function sorts an array of objects in ascending or descending order based
     * on a string or numerical property, maintaining array index associations.
     *
     * array $objArr: an array of objects to sort
     * string $orderBy: the string name of the property to group or sort by
     * int $d1: The direction to group or sort by. 0 == ascending, 1 == descending (optional)
     * string $sortBy: a string name of another property to sort on (optional)
     * int $d2: The direction to sort $sortBy. 0 == ascending, 1 == descending (optional)
     *
     * Final array is stored in the $objArr property
     **/
    function sortObjArray($objArr, $orderBy, $d1 = 0, $sortBy = NULL, $d2 = 0) {
        if (!is_array($objArr)) {
            $this->error('first argument is not an array');
        }
        $this->property = $orderBy;
        $this->direction = $d1;
        $this->orderBy = $orderBy;
        uasort($objArr, array($this, "cmp"));
        $this->objArr = $objArr;
        if ($sortBy!==NULL) {
            $holding = array();
            $this->property = $sortBy;
            $this->direction = $d2;
            foreach ($objArr as $key=>$val) {
                $holding[$val->{$orderBy}][$key] = $val;
            }
            $this->objArr = array();
            foreach ($holding as $key=>$val) {
                uasort($val, array($this, "cmp"));
                foreach($val as $k=>$v) {
                    $this->objArr[$k] = $v;
                }
            }
        }
    }

    /**
     * callback function required by sortObjArray() (private)
     **/
    function cmp ($a, $b) {
        if (!is_object($a) || !is_object($b)) {
            $this->error("array contains non-objects, must contain only consistant objects");
        }
        if (!isset($a->{$this->property}) || !isset($b->{$this->property})) {
            $this->error("'".$this->property."' is not a valid property");
        }
     // the following if statement does not work, means that it does not give a descending sort, dont know the reason at
        /*   if ($this->direction == 1) {
        	$tmp = $a;
            $a = $b;
            $b = $tmp;
        } */
        if (is_numeric($a->{$this->property}) && is_numeric($b->{$this->property})) {
            if ($a->{$this->property} == $b->{$this->property}) return 0;
            if ($this->direction == 1) {	
            	return ($a->{$this->property} < $b->{$this->property}) ? 1 : -1;
            } else {
            	return ($a->{$this->property} < $b->{$this->property}) ? -1 : 1;
            }
        } else {
            if ($this->direction == 1) {
        		return strcasecmp($b->{$this->property}, $a->{$this->property});
            } else {
            	return strcasecmp($a->{$this->property}, $b->{$this->property});
            }
        }
    }

    /**
     * error handler (private)
     **/
    function error($msg) {
        $trace = debug_backtrace();
        $trace = $trace[count($trace)-1];
        echo "<pre>\n\n[FATAL] ".get_class($this)."::".$trace['function']."[".$trace['line']."] - ".$msg."\n\n";
        var_dump($trace);
        echo "</pre>";
        die();
    }
}