<?php

require_once('class.db_management.inc');

/**
 * utility class for frequently needed user information 
 */
class HolidayCache {

	/**
	 * cache of holiday table snapshot:
	 * 	key = Holiday_ID
	 *		value = db_fetchmode_object
	 */
	var $holiday = array();
	
	var $monthName = array(1=>'J&auml;nner', 'Februar', 'M&auml;rz', 'April', 'Mai', 'Juni', 'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember', 0=>'Tag nach Ostern');

	function HolidayCache() {
    	$holiday = &$this->holiday;

		// fetch table rows
		$db = &new DB_Management;
		$db->open_db();
		$res = &$db->connection->query('select * from Holiday order by if(month, month, 99), day');
		while($row = &$res->fetchRow(DB_FETCHMODE_OBJECT)) {
			$row->key = 100*$row->month + $row->day;
			$holiday[$row->key] = &$row;
		}
		$db->close_db();
	}

	function save($key, $day, $month, $description) {
		$db = &new DB_Management;
		$db->open_db();
		if($key) {
			$stmt = &$db->connection->prepare('update Holiday set day=?, month=?, description=? where 100*month+day=?');
			$db->connection->execute(&$stmt, array($day, $month, $description, $key));
		} else {
			$stmt = &$db->connection->prepare('insert into Holiday(day, month, description) values (?,?,?)');
			$db->connection->execute(&$stmt, array($day, $month, $description));
		}
		$db->close_db();
	}
	
	function delete($key) {
		$db = &new DB_Management;
		$db->open_db();
		$stmt = &$db->connection->prepare('delete from Holiday where 100*month+day=?');
		$db->connection->execute(&$stmt, array($key));
		$db->close_db();
	}

	function isHoliday($date) {
		if(isset($this->holiday[(int) date('md', $date)])) {
			return true;
		}
		$fromEaster = Calendar::diff($date, Calendar::easter_date(date('Y', $date)));
		if($fromEaster >=0 && $fromEaster < 100 && isset($this->holiday[$fromEaster])) {
			return true;
		}
		return false;
	}

	function previousWorkDay($date) {
		while(Calendar::isWeekend($date) || $this->isHoliday($date)) {
			$date = Calendar::add($date, -1);
		}
		return $date;
	}

}

?>