<?php

require_once('class.db_management.inc');
require_once('class.util.inc');
require_once('class.UserCache.php');
require_once('class.ProjectCache.php');

class PerformanceCache {

	var $performance; // array of queried objects
	var $sumDuration; // total of queried objects->Duration in seconds

	var $projectCache;
	var $userCache;
	var $sessionUser; // object for logged in user
	var $privledgedUser; // boolean

	/**
	 * selection criteria
	 */
	var $Project_ID;
	var $User_ID;
	var $Perf_Type_ID;
	var $ProjectRole_ID;
	var $BeginDate;
	var $EndDate;
	var $subProjectFlag; // boolean
	var $deactivatedFlag; // boolean

	function PerformanceCache($sessionUser_ID=NULL) {
		$this->userCache = &new UserCache;
		$this->sessionUser = &$this->userCache->user[$sessionUser_ID];
		$this->priviledgedUser = ! $sessionUser_ID || in_array($this->sessionUser->UserGr_ID, array(1, 2));
		$this->projectCache = &new ProjectCache;
		if($sessionUser_ID) {
			$this->projectCache->addPM($sessionUser_ID);
		}
	}

	/**
	  * composes a select statement with where clause from the selection criteria
	  */
	function getSelectStatement($orderClause = 'order by WorkDate', $selectList='*, Performance.Description PerformanceDescription') {
		$clause = array('Performance.P_Assignment_ID=Project_Assignment.P_Assignment_ID');
		// priviledged user may see all entries; any user may see own entries;
		$no_restriction = $this->priviledgedUser || ($this->User_ID == $this->sessionUser->User_ID);
		if($this->Project_ID) {
			if($this->subProjectFlag) {
				$projectIDs = &$this->projectCache->getSubProjects($this->Project_ID, $no_restriction ? NULL : $this->sessionUser->User_ID);
				$clause[] = $projectIDs ? 'Project_ID in ('.implode(',', $projectIDs).')' : 'Project_ID<0';
			}
			else {
				$may_see = $no_restriction || $this->projectCache->isPM($this->Project_ID, $this->sessionUser->User_ID);
				$clause[] = $may_see ? 'Project_ID='.$this->Project_ID : 'Project_ID<0';
			}
		}
		elseif(! $no_restriction) {
			$projectIDs = &$this->projectCache->getSubProjects(0, $this->sessionUser->User_ID);
				$clause[] = $projectIDs ? 'Project_ID in ('.implode(',', $projectIDs).')' : 'Project_ID<0';
		}
		if($this->User_ID) {
			$clause[] = 'User_ID='.$this->User_ID;
		}
		if($this->Perf_Type_ID) {
			$clause[] = 'Perf_Type_ID='.$this->Perf_Type_ID;
		}
		if($this->Project_Role_ID) {
			$clause[] = 'ProjectRole_ID='.$this->ProjectRole_ID;
		}
		if($this->BeginDate && $this->EndDate) {
			$clause[] = 'WorkDate between \''.$this->BeginDate.'\' and \''.$this->EndDate.'\'';
		}
		if(! $this->deactivatedFlag) {
			$clause[] = 'Performance.Status=\'A\'';
		}
		return 'select '.$selectList.' from Performance join Project_Assignment where '.implode(' and ', $clause).' '.$orderClause;
	}
	
	/**
	 * performs a query according to the selection criteria and stores result objects in $this->performance array
	 */
	function query() {
		$performance = &$this->performance;
		$sumDuration = &$this->sumDuration;
		$performance = array();
		$sumDuration = 0;
		// fetch table rows
		$db = &new DB_Management;
		$db->open_db();
		$queryString = $this->getSelectStatement();
		echo "<!-- $queryString -->\n";
		$res = &$db->connection->query($queryString);
		echo "<!-- ".Util::checkDBresult(&$res)." -->\n";
		while($row = &$res->fetchRow(DB_FETCHMODE_OBJECT)) {
			$performance[] = &$row;
			$sumDuration += Time::str2sec($row->Duration);
		}
		$db->close_db();
	}
	
	/**
	  * performs a query according to the selection criteria and returns the sum of the Duration field in seconds
	  */ 
	function querySumDuration() {
		$db = &new DB_Management;
		$db->open_db();
		$queryString = $this->getSelectStatement(NULL, 'sum(time_to_sec(Duration))');
		echo "<!-- $queryString -->\n";
		$res = &$db->connection->query($queryString);
		$row = &$res->fetchRow(DB_FETCHMODE_ORDERED);
		$db->close_db();
		return $row[0];
	}
	
}

?>