<?php

require_once('class.db_management.inc');

/**
 * utility class for frequently needed project information 
 */
class ProjectCache {

	/**
	 * cache of project table snapshot:
	 * 	key = Project_ID
	 *		value = db_fetchmode_object
	 *		value->childs = array of subproject IDs
	 *		value->PMs = array with project managers as key
	 */
	var $project = array();

	function ProjectCache($condition="order by ProjectName") {
    	$project = &$this->project;

		// fetch table rows
		$db = &new DB_Management;
		$db->open_db();
		$res = &$db->connection->query('select * from Project '.$condition);
		while($row = &$res->fetchRow(DB_FETCHMODE_OBJECT)) {
			$project[$row->Project_ID] = &$row;
			$row->childs = array();
		}
		$db->close_db();
		// fill ...->childs with subproject IDs
		foreach($project as $key=>$value) {
      		$project[$value->HigherProject_ID]->childs[] = $key;
		}
	}

	function addPM($User_ID, $ProjectRole_ID=1) {
    	$project = &$this->project;
		$db = &new DB_Management;
		$db->open_db();
		$res = &$db->connection->query("select Project_ID from Project_Assignment where User_ID=$User_ID and ProjectRole_ID=$ProjectRole_ID");
		while($row = &$res->fetchRow(DB_FETCHMODE_OBJECT)) {
			$project[$row->Project_ID]->PMs[$User_ID] = 1;
		}
		$db->close_db();
	}

	/**
	 * returns the full path of a project (relative to a given project if specified)
	 */
	function getPath($Project_ID, $relative_ID=0, $separator='/') {
		return implode($separator, $this->getPathArray($Project_ID, $relative_ID));
	}

	/**
	 * returns an array with the ancestor project names as elements (relative to a given project if specified)
	 */
	function &getPathArray($Project_ID, $relative_ID=0) {
		if($Project_ID && ($Project_ID != $relative_ID)) {
			$project = &$this->project[$Project_ID];
			$pathArray = &$this->getPathArray($project->HigherProject_ID, $relative_ID);
			$pathArray[] = $project->ProjectName;
			return $pathArray;
		}
		else {
			return array();
		}
	}

	/**
	  * returns an array of project IDs of the given project and all descendant projects
	  * if user ID is given, restrict to projects where user is PM
	  */
	function &getSubProjects($Project_ID, $User_ID=NULL) {
		$sub = array();
		$this->addSubProjects($Project_ID, &$sub, $User_ID);
		return $sub;
	}
	
	/**
	 * adds the project IDs of the given project and all descendant projects to the given array
	 */
	function addSubProjects($Project_ID, &$sub, $User_ID) {
		if(is_null($User_ID) || $this->isPM($Project_ID, $User_ID)) {
			$sub[] = $Project_ID;
		}
		foreach($this->project[$Project_ID]->childs as $child) {
			$this->addSubProjects($child, &$sub, $User_ID);
		}
	}

	/**
	 * returns whether user is PM of given project or some of its ancestor projects
	 */
	function isPM($Project_ID, $User_ID) { // boolean
		if($Project_ID == 0) {
			return FALSE;
		}
		$project = &$this->project[$Project_ID];
		if($project->PMs[$User_ID]) {
			return TRUE;
		}
		return $this->isPM($project->HigherProject_ID, $User_ID);
	}

	/**
	 * returns the book back date of the given project or its closest ancestor for which one is defined
	 */
	function getBookBackDate($Project_ID) { // 'yyyy-mm-dd' or null
		if($Project_ID == 0) {
			return NULL;
		}
		$project = &$this->project[$Project_ID];
		if($project->BookBackDate && $project->BookBackDate != '0000-00-00') {
			return $project->BookBackDate;
		}
		return $this->getBookBackDate($project->HigherProject_ID);
	}

}

?>