<?
  // include files
  
  // PEAR DB: a unified API for accessing SQL-databases (http://pear.php.net)
  // the pear base directory must be in the include_path
  require_once("DB.php");
  
  // the includepath of db_info.inc must be above the document path,
  // so that the file is not attainable from outside
  $new_offset = substr($offset,0,-9);
  // include the database connection information
  include_once($offset."../secrets/db_info.inc");
  
  // include the error management class to announces errors
  include_once($offset."class.error_management.inc");
  
  /**
   * Database layer abstraction.
   * This class provides all necessary methods to work with a specified database.
   *
   * @author Martin Jobst (JMF)
   * @version 1.0 beta 2002/02/06
   */
  class DB_Management extends DB
  {
   /**
    * DB object of the current database connection.
    *
    * @type DB
    */
    var $connection;
  
  /**
   * open a database connection
   *
   * @returns void
   */
  function open_db()
  {
    if (!isset($this->connection)) {

      /* Data Source Name (DSN): This is the universal connection string
       * The format of the supplied DSN is in its fullest form:
       * phptype(dbsyntax)://username:password@protocol+hostspec/database
       *
       * phptype: Database backend used in PHP (mysql, odbc etc.)
       * dbsyntax: Database used with regards to SQL syntax etc.
       * protocol: Communication protocol to use (tcp, unix etc.)
       * hostspec: Host specification (hostname[:port])
       * database: Database to use on the DBMS server
       * username: User name for login
       * password: Password for login
       */
      $data_source_name=DBTYPE."://".DBUSER.":".DBPASSWORD."@".DBHOST."/".DBNAME;
      
      // DB::connect will return a DB object on success or a DB Error object on error
      $this->connection = DB::connect($data_source_name, true);
      // With DB::isError you can differentiate between an error or a valid connection.
      if (DB::isError($this->connection)) {
        $error = new Error_Management;
        $error->error_page('',$this->connection->getMessage());
        die();
      }
    }
  }

  /**
   * close the current database connection
   *
   * @returns void
   */
  function close_db()
  {
    $this->connection->disconnect();
    if (DB::isError($this->connection)) {
      $error = new Error_Management;
      $error->error_page('',$this->connection->getMessage());
      die();
    }
  }


  /**
   * execute a SQL-Querystring to the database.
   *
   * @param string query: the SQL-Querystring which will be executed
   * @returns mixed
   */
  function execute_query($query)
  {
    // DEBUG must be FALSE while applying the class dump, else a mistake (Header) occurs
    if (DEBUG) {
      echo "<!-- query = ".$query." -->\n";
    }
    /* If the query is a "SELECT", $db->query will return a DB_result on success.
     * Else it simply will return a DB_OK. On failure it will return a DB Error object.
     */
    $result=$this->connection->query($query);

    if (DB::isError($result)) {
      $error = new Error_Management;
      $error->error_page('',$result);
      // $error->error_page('',$result->getMessage());
      die();
    }
    // DEBUG must be FALSE while applying the class dump, else a mistake (Header) occurs
    if (DEBUG) {
      echo "<!-- result = ".$result." -->\n";
    }
    return $result;
  }

   
  /**
   * fetch the rows from the result of the query and returns an ordered array.
   * The order is taken from the select statment.
   *
   * @param DB_result result: the result of the query
   * @returns mixed
   */
  function fetch_array($result)
  {
    $row = $result->fetchRow(DB_FETCHMODE_ORDERED);
    if (DB::isError($row)) {
      $error = new Error_Management;
      $error->error_page('',$row->getMessage());
      die();
    }
    return $row;
  }

   
  /**
   * fetch the rows from the result of the query and
   * returns DB_row object with column names as properties.
   *
   * @param DB_result result: the result of the query
   * @returns mixed
   */
  function fetch_row($result)
  {
    $row = $result->fetchRow(DB_FETCHMODE_OBJECT);
    if (DB::isError($row)) {
      $error = new Error_Management;
      $error->error_page('',$row->getMessage());
      die();
    }

    return $row;
  }

  /**
   * fetch the rows from the result of the query and
   * returns an associative array with column names as array keys.
   *
   * @param DB_result result: the result of the query
   * @returns mixed
   */
  function fetch_associative_array($result)
  {
    $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
    if (DB::isError($row)) {
      $error = new Error_Management;
      $error->error_page('',$row->getMessage());
      die();
    }
    return $row;
  }



  /**
   * fetch the first column of the first row from a query.
   *
   * @param string query: the SQL-Querystring which will be executed
   * @returns DB_Result
   */
  function get_single_value($query)
  {
    // DEBUG must be FALSE while applying the class dump, else a mistake (Header) occurs
    if (DEBUG) {
      echo "<!-- query = ".$query." -->\n";
    }
    $result = $this->connection->getOne($query);
    if (DB::isError($result)) {
      $error = new Error_Management;
      $error->error_page('',$result);
      // $error->error_page('',$result->getMessage());
      die();
    }
    // DEBUG must be FALSE while applying the class dump, else a mistake (Header) occurs
    if (DEBUG) {
      echo "<!-- result = ".$result." -->\n";
    }
    return $result;
  }


  /**
   * returns the number of rows of the table which ones fill the query.
   *
   * @param string table: the table for the query
   * @param string where: the where statement of the SQL-Querystring
   * @returns mixed
   */
  function get_count($table, $where)
  {
    $query="SELECT COUNT(*) FROM ".$table." WHERE ".$where;
    // DEBUG must be FALSE while applying the class dump, else a mistake (Header) occurs
    if (DEBUG) {
      echo "<!-- query = ".$query." -->\n";
    }
    $result = $this->connection->getOne($query);
    if (DB::isError($result)) {
      $error = new Error_Management;
      $error->error_page('',$result->getMessage());
      die();
    }
    // DEBUG must be FALSE while applying the class dump, else a mistake (Header) occurs
    if (DEBUG) {
      echo "<!-- result=$result -->\n";
    }
    return $result;
  }


  /**
   * ruturns the highest ID-number of a field of a table.
   * This method is used to get the new ID-number of a insert-operation.
   *
   * @param string table: the table for the query
   * @param string field_name: the name of the field from which the highest ID-number is searched
   * @returns int
   */
  function get_insert_id($table, $field_name)
  {
    $query="SELECT MAX($field_name) FROM $table" ;
    $last_inserted_id = $this->connection->getOne($query);
    if (DB::isError($last_inserted_id)) {
      $error = new Error_Management;
      $error->error_page('',$last_inserted_id->getMessage());
      die();
    }
    if ($last_inserted_id< 1) {
      $last_inserted_id= -1;
    }
    return $last_inserted_id;
  }

  /**
   * returns the number of rows affected by a data manipulation query ("INSERT", "UPDATE" or "DELETE").
   *
   * @returns int
   */
  function get_affected_rows()
  {
    $count_affected_rows = $this->connection->affectedRows();
    if (DB::isError($count_affected_rows)) {
      $error = new Error_Management;
      $error->error_page('',$count_affected_rows->getMessage());
      die();
    }
    return $count_affected_rows;
  }


  /**
   * returns the the number of columns in a result set.
   *
   * @param DB_result result: the result of the query
   * @returns int
   */
  function number_of_fields($result)
  {
    $number_of_fields=$result->numCols();
    return $number_of_fields;
  }
  

  /**
   * returns the fieldname of the given fieldnumber from a resultset.
   *
   * @param DB_result result: the result of the query
   * @param int index: the number of the field in the resultset
   * @returns string
   */
  function get_field_name($result,$index)
  {
    $tabel_info=$result->tableInfo();
    $field_name = $tabel_info[$index]["name"];
    return $field_name;
  }

  /**
   * prepares a string for storing in the database.
   *
   * @param string string_to_encode: the string which should be encoded
   * @returns string
   */
  function encode_string($string_to_encode)
  {
    // trim - strip whitespace from the beginning and end of a string
    $str = trim($string_to_encode);
    $encoded_string = str_replace("'", "'''", $str);
    return $encoded_string;
  }

  /**
   * prepares a string for storing in the database.
   *
   * @param string string_to_encode: the string which should be encoded
   * @returns string
   */
  function encode_string_not_always_working($string_to_encode)
  {
    // trim - strip whitespace from the beginning and end of a string
    $str = trim($string_to_encode);
	return $this->connection->escapeSimple($str);
  }  
 
  function sqlString($value) {
  	if(is_null($value)) {
  		return 'null';
  	}
  	if(is_numeric($value)) {
  		return $value;
  	}
  	if(is_string($value)) {
  		return "'".$this->encode_string($value)."'";
  	}
  }

  function sqlString_not_always_working($value) {
  	return $this->connection->quoteSmart($value);
  }

}
?>
