<?php

  // include class PM_Common for common used methods
  require_once($offset."class.pm_common.inc");

  // require_once($offset."class.project.inc");
 
 
  /**
   * The class Dump is used to dump a table or selected attributes of tables of the database.
   *
   * @author Martin Jobst (JMF)
   * @version 1.0 beta 2002/02/11
   */
  class Dump
  {

    /**
     * simple constructor to generate a Dump object
     * @returns void
     */
    function Dump() { }
    
    
    /**
     * Returns the resultset with the workdates, the start- and the endtimes, the durations, the effort types,
     * the projectnames, the driven kilometers, the locations, the usernames and the project roles
     * of the efforts that fulfill the selection criteria.
     *
     * @param int p_assignment_array: an array of project assignment IDs of the efforts
     * @param string begin_date: begin date of the period of time, in which one is queried,
     *  whether the effort was carried out in this period
     * @param string end_date: end date of the period of time, in which one is queried,
     *  whether the effort was carried out in this period
     * @param string status: determines whether the efforts which are queried for are "a"ctive or "n"ot active
     * @returns DB_result
     */
    function get_full_effort_dump($p_assignment_array,$begin_date,$end_date,$status)
    {
      if (!empty($p_assignment_array)) {
        $db = new DB_Management;
        $db->open_db();
        
        $p_assignment_string=implode(",",$p_assignment_array);

        $query = "SELECT WorkDate,StartTime,EndTime,Duration,PeTy.Perf_Type_Name AS Performance_Type, Proj.ProjectName AS Project_Name, "
               . " km, Performance.Description, Loca.LocationName AS Location_Name, CONCAT(User.FirstName, ' ', User.LastName) AS User_Name, "
               . " PrRo.P_RoleName AS Project_Role FROM Performance AS Performance, Project_Assignment AS PrAs, Location AS Loca, "
               . " Performance_Type AS PeTy, Project AS Proj, Project_Role AS PrRo, User "
               . " WHERE Performance.P_Assignment_ID=PrAs.P_Assignment_ID AND PrRo.ProjectRole_ID = PrAs.ProjectRole_ID"
               . " AND Performance.Location_ID=Loca.Location_ID AND Performance.Perf_Type_ID=PeTy.Perf_Type_ID AND PrAs.User_ID=User.User_ID"
               . " AND PrAs.Project_ID = Proj.Project_ID AND Performance.P_Assignment_ID IN ($p_assignment_string)"
               . " AND Performance.WorkDate BETWEEN '$begin_date' AND '$end_date' ";

        if ((strcasecmp($status,'A')==0) || (strcasecmp($status,'N')==0)) {
         $query = $query . " AND Performance.Status = '$status' ";
        }

        $result = $db->execute_query($query);
      } else {
        $result = "";
      }
      $db->close_db();
      return $result;
    } // end function get_full_effort_dump
    

    /**
     * Returns the resultset with all attributes of the efforts that fulfill the selection criteria.
     *
     * @param int p_assignment_array: an array of project assignment IDs of the efforts
     * @param string begin_date: begin date of the period of time, in which one is queried,
     *  whether the effort was carried out in this period
     * @param string end_date: end date of the period of time, in which one is queried,
     *  whether the effort was carried out in this period
     * @param string status: determines whether the efforts which are queried for are "a"ctive or "n"ot active
     * @returns DB_result
     */
    /*
    function get_performance_dump($p_assignment_array,$begin_date,$end_date,$status)
    {
      if (!empty($p_assignment_array)) {
        $db = new DB_Management;
        $db->open_db();
        
        $p_assignment_string=implode(",",$p_assignment_array);
        
        $query = "SELECT Performance.WorkDate,Performance.Duration,PeTy.Perf_Type_Name,Perf.Description,CONCAT(User.FirstName,' ',User.LastName),Perf.Km  FROM Performance AS Perf, Project_Assignment AS PrAs, Location AS Loca, User, "
               . " Performance_Type AS PeTy WHERE Perf.P_Assignment_ID=PrAs.P_Assignment_ID AND "
               . " Perf.Location_ID=Loca.Location_ID AND Perf.Perf_Type_ID=PeTy.Perf_Type_ID "
               . " AND PrAs.P_Assignment_ID = Perf.P_Assignment_ID AND PrAs.User_ID = User.User_ID "
               . " AND Perf.P_Assignment_ID IN ($p_assignment_string)"
               . " AND Perf.WorkDate BETWEEN '$begin_date' AND '$end_date' ";

        if ((strcmp($status,'A')==0) || (strcmp($status,'N')==0)) {
         $query = $query . " AND Performance.Status = '$status' ";
        }
        
        $result = $db->execute_query($query);
      } else {
        $result = "";
      }
      $db->close_db();
      return $result;
    } // end function get_effort_dump
        
    */

    /**
     * Returns the resultset with all attributes of the given table.
     *
     * @param string table: the table from which one the dump is made
     * @returns DB_result
     */
    function get_table_dump($table)
    {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT * FROM $table";
      $result = $db->execute_query($query);

      $db->close_db();
      return $result;
    } // end function get_table_dump
    
	/* obsolete...
    function get_performance_dump($p_assignment_id_array,$begin_date,$end_date,$perf_status,$deactivatedstatus) {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT P.WorkDate AS Datum, " .
               "P.Duration AS Dauer, " .
               "PT.Perf_Type_Name AS Leistungstyp, ". 
               // Projektname !!!
               "PA.Project_ID AS Projekt, " .
               "P.Description AS Beschreibung, " .
               "CONCAT(U.FirstName,' ',U.LastName) As Mitarbeiter, " .
               "P.Km AS Km, " .
               "P.Status AS Status " .
               "FROM Performance AS P, Performance_Type AS PT, Bill_Type AS BT, " .
               "Location AS L, User AS U, Project_Assignment AS PA " .
               "WHERE P.Perf_Type_ID = PT.Perf_Type_ID AND " .
               "P.Bill_Type = BT.Bill_Type_ID AND " .
               "L.Location_ID = P.Location_ID AND " .
               "PA.P_Assignment_ID = P.P_Assignment_ID AND " .
               "PA.User_ID = U.User_ID AND " .
               "P.WorkDate BETWEEN '$begin_date' AND '$end_date'";
      // echo "deactivatedstatus = `$deactivatedstatus'\n";
      if ($deactivatedstatus == "no") {
        $query .= " AND P.Status = 'A'";
      }

      // echo $query;

      $result = $db->execute_query($query);
      $db->close_db();

      if (strcmp($db->get_field_name($result,3),"Projekt")==0) {
        
      }

      return $result;
    }
	*/

    function get_performance_dump($selected_user_id,$begin_date,$end_date,$perf_status,$deactivatedstatus) {
      $query = "SELECT P.WorkDate AS Datum, " .
               "P.Duration AS Dauer, " .
               "PT.Perf_Type_Name AS Leistungstyp, ". 
               // Projektname !!!
               "PA.Project_ID AS Projekt, " .
               "P.Description AS Beschreibung, " .
               "CONCAT(U.FirstName,' ',U.LastName) As Mitarbeiter, " .
               "P.Km AS Km, " .
               "P.Status AS Status " .
               "FROM Performance AS P, Performance_Type AS PT, User AS U, Project_Assignment AS PA ".
//               "FROM Performance AS P, Performance_Type AS PT, Bill_Type AS BT, " .
//               "Location AS L, User AS U, Project_Assignment AS PA " .
               "WHERE P.Perf_Type_ID = PT.Perf_Type_ID AND " .
//               "P.Bill_Type = BT.Bill_Type_ID AND " .
//               "L.Location_ID = P.Location_ID AND " .
               "PA.P_Assignment_ID = P.P_Assignment_ID AND " .
               "PA.User_ID = U.User_ID AND " .
               "P.WorkDate BETWEEN '$begin_date' AND '$end_date'";

	  if($selected_user_id != -1) {
	  	$query .= " AND PA.User_ID = '$selected_user_id'";
	  }

      // echo "deactivatedstatus = `$deactivatedstatus'\n";
      if ($deactivatedstatus == "no") {
        $query .= " AND P.Status = 'A'";
      }

      // echo $query;

      $db = new DB_Management;
      $db->open_db();
      $result = $db->execute_query($query);
      $db->close_db();

      return $result;
		
    }

    /**
     * Make the dump of the given resultset in the file "table.format"
     *
     * @param string table: the name of the table from which one the dump was made;
       this is the name of the dump file too
     * @param DB_result result_set: the resulset of the dump selection
     * @param string delimiter: to delimit the result values in the dump file
     * @param string format: to format of the dump file
     * @returns void
     */
    function make_dump($table, $result_set, $delimiter, $format)
    {
      $db = new DB_Management;
      $db->open_db();

      Header("Content-Disposition: filename=$table.$format");
      Header("Content-Type: text/plain");

      $number_of_fields = $db->number_of_fields($result_set);

      for ($ctr = 0; $ctr < $number_of_fields; $ctr++) {
        echo "\"".$db->get_field_name($result_set, $ctr)."\"";
        if ($ctr < ($number_of_fields - 1)) {
          echo $delimiter;
        }
      }
      echo "\n";
      while ($row = $db->fetch_array($result_set)) {
        for ($ctr = 0; $ctr < ($number_of_fields); $ctr++) {
          if (strcmp($table,"Performance")==0 && $ctr == 3) {
            $row[$ctr] = Project::get_full_project_path($row[$ctr]);
          }
          $field = $row[$ctr];
          $field = str_replace("\n","\\n",$field);
          // $field = str_replace("\r","\\r",$field);
          $field = str_replace("\r","",$field);
          echo "\"$field\"";
          if ($ctr < ($number_of_fields - 1)) {
            echo $delimiter;
          }
        }
        echo "\n";
      }
      $db->close_db();
    } // end function make_dump

  }  // end Class Dump
?>
