<?php

  // include class PM_Common for common used methods
  include_once($offset."class.pm_common.inc");


  /**
   * The class Location is used for the administration of the information appertaining
   * to the location of a performance.
   * Also methods for various querys are available.
   *
   * @author Martin Jobst (JMF)
   * @version 1.0 beta 2002/02/12
   */
  class Location
  {
    /**
     * unique location number (ID) which is generated by the database
     * @type int
     */
    var $location_id;
    /**
     * name of the location
     * @type string
     */
    var $location_name;
    /**
     * description of the location
     * @type string
     */
    var $description;
    

    /**
     * simple constructor to generate a Location object
     * @returns void
     */
    function Location() { }


    /**
     * enhanced constructor to generate a Location object
     *
     * @param int location_id: location ID of the location which will be generated
     * @returns void
     */
    function Location($location_id='')
    {
      $this->location_id=$location_id;
    }


    /**
     * creates a new location and returns the ID of the new location
     *
     * @param Location location: the new location that in the database will be stored
     * @param int user_id_log: the user ID of the person who creates the new location entry
     * @returns int
     */
    function new_location($location, $user_id_log)
    {
      $check=$this->check_params($location, $user_id_log);
      if (strcmp($check,"") == 0)  {
        $db = new DB_Management;
        $db->open_db();
        $query ="INSERT INTO Location(LocationName, Description, User_ID_Log) VALUES (";
        $query = $query."'".$db->encode_string($location->location_name)."', ";
        $query = $query."'".$db->encode_string($location->description)."', ";
        $query = $query." $user_id_log )";

        $db->execute_query($query);

        if ($db->get_affected_rows() >0 ) {
          $new_location_id = $db->get_insert_id('Location','Location_ID');
        } else {
          $new_location_id=-1;
        }
        $db->close_db();
        return $new_location_id;
      } else {
        $error = new Error_Management;
        $error->error_page(0,$check);
        die();
      }
    } // end function new_location


    /**
     * updates the edited attributes of a location and returns "1" on success or "0" on error
     *
     * @param Location location: the edited location that in the database will be updated
     * @param int user_id_log: the user ID of the person who updates the edited location
     * @returns int
     */
    function edit_location($location, $user_id_log)
    {
      $check=$this->check_params($location, $user_id_log);
      if (($location->location_id>0) && (strcmp($check,"") == 0))  {
        $db = new DB_Management;
        $db->open_db();
        $query ="UPDATE Location SET ";
        $query = $query."LocationName='".$db->encode_string($location->location_name)."', ";
        $query = $query."Description='".$db->encode_string($location->description)."', ";
        $query = $query."User_ID_Log=$user_id_log ";
        $query = $query."WHERE Location_ID=$location->location_id ";

        $edit_result=$db->execute_query($query);
        $db->close_db();
        return $edit_result;
      } else {
        $error = new Error_Management;
        $error->error_page(0,$check);
        die();
      }
    } // end function edit_location


    /**
     * deletes a location and returns "1" on success or "0" on error
     *
     * @param int location_id: location ID of the location which in the database will be deleted
     * @returns int
     */
    function delete_location($location_id)
    {
      if ($location_id > 0 ) {
        $query ="DELETE FROM Location WHERE Location_ID=$location_id";
        $db = new DB_Management;
        $db->open_db();
        $delete_result = $db->execute_query($query);
        $db->close_db();
        return $delete_result;
      } else {
        $error = new Error_Management;
        $error->error_page(120,"");
        die();
      }
    } // end function delete_location


    /**
     * returns the Location object with the given location ID
     *
     * @param int location_id: location ID of the Customer object which will be returned
     * @returns Location
     */
    function get_location($location_id)
    {
      if ($location_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT * FROM Location WHERE Location_ID= $location_id";
        $result = $db->execute_query($query);

        while ($row = $db->fetch_row($result)) {
          $this->location_id= $row->Location_ID;
          $this->location_name = $row->LocationName;
          $this->description = $row->Description;
        }
        $db->close_db();
        return $this;
      } else {
        $error = new Error_Management;
        $error->error_page(120,"");
        die();
      }
    } // end function get_location
    
    
    /**
     * returns all stored Location objects
     *
     * @param string orderby: to order the resultset by one or more attributes
     * @returns array
     */
    function get_locations($orderby='')
    {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT * FROM Location ";
      if (empty($orderby)) {
        $orderby="Location_ID";
      }
      $query = $query . " ORDER BY ". $orderby;
      $result = $db->execute_query($query);

      $i=1;
      while ($row = $db->fetch_row($result)) {
        $this->location_id= $row->Location_ID;
        $this->location_name = $row->LocationName;
        $this->description = $row->Description;
        $location[$i]=$this;
        $i++;
      }

      $db->close_db();
      return $location;
    } // end function get_locations
    

    /**
     * returns the name of the location
     *
     * @param int location_id: location ID of the location from which one the name is queried
     * @returns string
     */
    function get_location_name($location_id)
    {
      if ($location_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT LocationName FROM Location WHERE Location_ID= $location_id";
        $this->location_name = $db->get_single_value($query);

        $db->close_db();
        return $this->location_name;
      } else {
        $error = new Error_Management;
        $error->error_page(120,"");
        die();
      }
    } // function get_location_name


    /**
     * returns the description of the location
     *
     * @param int location_id: location ID of the location from which one the description is queried
     * @returns int
     */
    function get_description($location_id)
    {
      if ($location_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Description FROM Location WHERE Location_ID= $location_id";
        $this->description = $db->get_single_value($query);

        $db->close_db();
        return $this->description;
      } else {
        $error = new Error_Management;
        $error->error_page(120,"");
        die();
      }
    } // end function get_description
    
    
    /**
     * checks the validity of the attribute values of a Location object and
     * if the validity is ok then returns "" else returns the corresponding check message
     *
     * @param Location location: the Location object which has to be checked
     * @param int user_id_log: the user ID of the person which has to be checked
     * @returns string
     */
    function check_params($location, $user_id_log)
    {
      $check_message = i18n("F&uuml;r den/die Parameter ");
      $check_counter = 0;

      if (strcmp($location->location_name,"") == 0) {
        $check_message = $check_message . i18n("Arbeitsort");
        $check_counter++;
      }
      if ($user_id_log < 1) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Bearbeiter-Benutzer-ID");
        $check_counter++;
      }

      $check_message = $check_message . i18n(" wurden keine oder falsche Werte eingegeben! &Uuml;berpr&uuml;fen Sie ihre Eingabe nochmal!");

      if ($check_counter == 0) {
        $check_message="";
      }
      return $check_message;
    } // end function check_params


    /**
     * sets the name of a location and returns "1" on success or "0" on error
     *
     * @param int location_id: location ID of the location from which one the name is set
     * @param string location_name: the location name which is set
     * @returns int
     */
    function set_location_name($location_id, $location_name)
    {
      if (($location_id > 0) && (strlen($location_name) > 0)){
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Location SET LocationName='".$db->encode_string($location_name)."' WHERE Location_ID=$location_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->location_name =$location_name;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_location_name


    /**
     * sets the description of a location and returns "1" on success or "0" on error
     *
     * @param int location_id: location ID of the location from which one the description is set
     * @param string description: the description which is set
     * @returns int
     */
    function set_location_description($location_id, $description)
    {
      if ($user_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Location SET Description='".$db->encode_string($description)."' WHERE Location_ID=$location_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->description = $description;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_location_description
    
  }  // end class Location
?>
