<?php

  // include class PM_Common for common used methods
  include_once($offset."class.pm_common.inc");


  /**
   * The class Priority is used for the administration of the information appertaining to a project priority.
   * Also methods for various querys are available.
   *
   * @author Martin Jobst (JMF)
   * @version 1.0 beta 2002/02/11
   */
  class Priority
  {

    /**
     * unique project priority number (ID) which is generated by the database
     * @type int
     */
    var $priority_id;
    /**
     * type (name) of the project priority
     * @type string
     */
    var $priority_type;
    /**
     * description of the project priority
     * @type string
     */
    var $description;


    /**
     * simple constructor to generate a Priority object
     * @returns void
     */
    function Priority() { }
    

    /**
     * enhanced constructor to generate a Priority object
     *
     * @param int priority_id: priority ID of the project priority which will be generated
     * @returns void
     */
    function Priority($priority_id='')
    {
      $this->priority_id=$priority_id;
    }


    /**
    * creates a new project priority and returns the ID of the new priority
    *
    * @param Priority priority: the new project priority that in the database will be stored
    * @param int user_id_log: the user ID of the person who creates the new priority entry
    * @returns int
    */
    function new_priority($priority, $user_id_log)
    {
      $check=$this->check_params($priority, $user_id_log);
      if (strcmp($check,"") == 0)  {
        $db = new DB_Management;
        $db->open_db();
        $query ="INSERT INTO Priority(PriorityType, Description, User_ID_Log) VALUES (";
        $query = $query."'".$db->encode_string($priority->priority_type)."', ";
        $query = $query."'".$db->encode_string($priority->description)."', ";
        $query = $query." $user_id_log )";

        $db->execute_query($query);

       if ($db->get_affected_rows() >0 ) {
          $new_priority_id = $db->get_insert_id('Priority','Priority_ID');
        } else {
          $new_priority_id=-1;
        }
        $db->close_db();
        return $new_priority_id;
      } else {
        $error = new Error_Management;
        $error->error_page(0,$check);
        die();
      }
    } // end function new_priority
    
    
    /**
     * updates the edited attributes of a priority and returns "1" on success or "0" on error
     *
     * @param Priority priority: the edited priority that in the database will be updated
     * @param int user_id_log: the user ID of the person who updates the edited priority
     * @returns int
     */
    function edit_priority($priority, $user_id_log)
    {
      $check=$this->check_params($priority, $user_id_log);
      if (($priority->priority_id > 0) && (strcmp($check,"") == 0))  {
        $db = new DB_Management;
        $db->open_db();
        $query ="UPDATE Priority SET ";
        $query = $query."PriorityType='".$db->encode_string($priority->priority_type)."', ";
        $query = $query."Description='".$db->encode_string($priority->description)."', ";
        $query = $query."User_ID_Log=$user_id_log ";
        $query = $query."WHERE Priority_ID=$priority->priority_id ";

        $edit_result=$db->execute_query($query);
        $db->close_db();
        return $edit_result;
      } else {
        $error = new Error_Management;
        $error->error_page(0,$check);
        die();
      }
    } // end function edit_priority
    
    
    /**
     * deletes a project priority and returns "1" on success or "0" on error
     *
     * @param int priority_id: priority ID of the project priority which in the database will be deleted
     * @returns int
     */
    function delete_priority($priority_id)
    {
      if ($priority_id > 0 ) {
        $query ="DELETE FROM Priority WHERE Priority_ID=$priority_id";
        $db = new DB_Management;
        $db->open_db();
        $delete_result = $db->execute_query($query);
        $db->close_db();
        return $delete_result;
      } else {
        $error = new Error_Management;
        $error->error_page(140,"");
        die();
      }
    } // end function delete_priority


    /**
     * returns the Priority object with the given priority ID
     *
     * @param int priority_id: priority ID of the Priority object which will be returned
     * @returns Priority
     */
    function get_priority($priority_id)
    {
      if ($priority_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT * FROM Priority WHERE Priority_ID= $priority_id";
        $result = $db->execute_query($query);

        while ($row = $db->fetch_row($result)) {
          $this->priority_id= $row->Priority_ID;
          $this->priority_type = $row->PriorityType;
          $this->description = $row->Description;
        }
        $db->close_db();
        return $this;
      } else {
        $error = new Error_Management;
        $error->error_page(140,"");
        die();
      }
    } // end function get_priority

     
    /**
     * returns all stored Priority objects
     *
     * @param string orderby: to order the resultset by one or more attributes
     * @returns array
     */
    function get_priorities($orderby='')
    {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT * FROM Priority ";
      if (empty($orderby)) {
        $orderby="Priority_ID";
      }
      $query = $query . " ORDER BY ". $orderby;
      $result = $db->execute_query($query);

      $i=1;
      while ($row = $db->fetch_row($result)) {
        $this->priority_id= $row->Priority_ID;
        $this->priority_type = $row->PriorityType;
        $this->description = $row->Description;
        $priority[$i]=$this;
        $i++;
      }

      $db->close_db();
      return $priority;
    } // end function get_priorities
    

    /**
     * returns the priority type of the performance
     *
     * @param int priority_id: priority ID of the priority from which one the priority type is queried
     * @returns string
     */
    function get_priority_type($priority_id)
    {
      if ($priority_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT PriorityType FROM Priority WHERE Priority_ID= $priority_id";
        $priority_type = $db->get_single_value($query);

        $db->close_db();
        return $priority_type;
      }
      return "";
    } // function get_priority_type


    /**
     * returns the description of the priority
     *
     * @param int priority_id: priority ID of the priority from which one the description is queried
     * @returns int
     */
    function get_description($priority_id)
    {
      if ($priority_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Description FROM Priority WHERE Priority_ID= $priority_id";
        $description = $db->get_single_value($query);

        $db->close_db();
        return $description;
      } else {
        $error = new Error_Management;
        $error->error_page(140,"");
        die();
      }
    } // end function get_description
    
    
    /**
     * checks the validity of the attribute values of a Priority object and
     * if the validity is ok then returns "" else returns the corresponding check message
     *
     * @param Priority priority: the Priority object which has to be checked
     * @param int user_id_log: the user ID of the person which has to be checked
     * @returns string
     */
    function check_params($priority, $user_id_log)
    {
      $check_message = i18n("F&uuml;r den/die Parameter ");
      $check_counter = 0;

      if (strcmp($priority->priority_type,"") == 0) {
        $check_message = $check_message . i18n("Priorit&auml;t");
        $check_counter++;
      }
      if ($user_id_log < 1) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Benutzer-ID");
        $check_counter++;
      }
      $check_message = $check_message . i18n(" wurden keine oder falsche Werte eingegeben! &Uuml;berpr&uuml;fen Sie ihre Eingabe nochmal!");
      if ($check_counter == 0) {
        $check_message="";
      }
      return $check_message;
    } // end function check_params
    
    
    /**
     * sets the priority type of a priority and returns "1" on success or "0" on error
     *
     * @param int priority_id: priority ID of the priority from which one the priority type is set
     * @param string priority_type: the priority type which is set
     * @returns int
     */
    function set_priority_type($priority_id, $priority_type)
    {
      if (($priority_id > 0) && (strlen($priority_type) > 0)){
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Priority SET PriorityType='".$db->encode_string($priority_type)."' WHERE Priority_ID=$priority_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_priority_type


    /**
     * sets the description of a priority and returns "1" on success or "0" on error
     *
     * @param int priority_id: priority ID of the priority from which one the description is set
     * @param string description: the description which is set
     * @returns int
     */
    function set_description($priority_id, $description)
    {
      if ($user_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Priority SET Description='".$db->encode_string($description)."' WHERE Priority_ID=$priority_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } //end  function set_description

  } // end class Priority
?>
