<?php

  // include class PM_Common for common used methods
  include_once($offset."class.pm_common.inc");

  
  /**
   * The class Project is used for the administration of the information appertaining to a project.
   * Also methods for various querys are available.
   *
   * @author Martin Jobst (JMF)
   * @version 1.0 beta 2002/02/11
   */
  class Project {

    /**
    * unique project identification number (ID) which is generated by the database
    * @type int
    */
    var $project_id;
    /**
    * name of the project
    * @type string
    */
    var $project_name;
    /**
    * order number of the project in a level of hierarchy of the projects
    * @type int
    */
    var $order_number=0;
    /**
    * project identification number of the parent project
    * @type int
    */
    var $higher_project_id;
    /**
    * scheduled time to work in manhours
    * @type real
    */
    var $target_working_time;
    /**
    * scheduled date to begin the project
    * @type string
    */
    var $target_start_date;
    /**
    * real date to begin the project
    * @type string
    */
    var $real_start_date;
    /**
    * scheduled date to finalize the project
    * @type string
    */
    var $target_end_date;
    /**
    * real date to finalize the project
    * @type string
    */
    var $real_end_date;
    /**
    * estimated progress of the project
    * @type int
    */
    var $progress;
    /**
    * status ID of the project
    * @type int
    * @see class.status.inc
    */
    var $status_id;
    /**
    * priority ID of the project
    * @type int
    * @see class.priority.inc
    */
    var $priority_id;
    /**
    * partner for the project; that is another company or institution
    * @type string
    */
    var $p_partner;
    /**
    * description of the project
    * @type string
    */
    var $description;

    /**
    * name of the project in the Concurrent Versions System (CVS)
    * @type string
    */
    var $cvs_name;
    /**
    * possible branch of the project in the Concurrent Versions System (CVS)
    * @type string
    */
    var $cvs_branch;
    /**
    * remarkable details of the project
    * @type string
    */
    var $remark;


    /**
     * the project name cache
     */
    var $pn_cache;

    var $proj_cache;


    /**
     * enhanced constructor to generate a project object
     *
     * @param int project_id: project ID of the project which will be generated
     * @returns void
     */
    function Project($project_id='')
    {
        $this->project_id=$project_id;
        $this->pn_cache = array();
        $this->proj_cache = array();
    }


    /**
     * creates a new project and returns the ID of the new project
     *
     * @param Project project: the new project that in the database will be stored
     * @param int user_id_log: the user ID of the person who creates the new project
     * @returns int
     */
    function new_project($project, $user_id_log)
    {
      $check=$this->check_params($project,$user_id_log);
      if (strcmp($check,"") == 0)  {
        $db = new DB_Management;
        $db->open_db();
        $query ="INSERT INTO Project (ProjectName, Order_Number, HigherProject_ID, Report_Flag, TargetWorkingTime, TargetStartDate, RealStartDate, TargetEndDate, RealEndDate, Progress, Status_ID, Priority_ID, P_Partner, Description, BookBackDate, Cvs_Name, Cvs_Branch, Remark, User_ID_Log) VALUES (";
        $query = $query."'".$db->encode_string($project->project_name)."', ";
        $query = $query."$project->order_number, ";
        $query = $query."$project->higher_project_id, ";
        $query = $query.($project->report_flag ? 1 : 0).', ';
        $query = $query."'".$db->encode_string($project->target_working_time)."', ";
        $query = $query."'".$db->encode_string($project->target_start_date)."', ";
        $query = $query."'".$db->encode_string($project->real_start_date)."', ";
        $query = $query."'".$db->encode_string($project->target_end_date)."', ";
        $query = $query."'".$db->encode_string($project->real_end_date)."', ";
        $query = $query.$db->sqlString($project->progress).', ';
        $query = $query."$project->status_id, ";
        $query = $query."$project->priority_id, ";
        $query = $query."'".$db->encode_string($project->p_partner)."', ";
        $query = $query."'".$db->encode_string($project->description)."', ";
        $query = $query."'".$db->encode_string($project->bookback_date)."', ";
        $query = $query."'".$db->encode_string($project->cvs_name)."', ";
        $query = $query."'".$db->encode_string($project->cvs_branch)."', ";
        $query = $query."'".$db->encode_string($project->remark)."', ";
        $query = $query."$user_id_log )";
        $db->execute_query($query);
        if ($db->get_affected_rows() >0 ) {
          $new_project_id = $db->get_insert_id('Project','Project_ID');
        } else {
          $new_project_id=-1;
        }
        $db->close_db();
        return $new_project_id;
      } else {
          $error = new Error_Management;
          $error->error_page(0,$check);
          die();
      }
    } // end function new_project


    /**
     * updates the edited attributes of a project and returns "1" on success or "0" on error
     *
     * @param Project project: the edited project that in the database will be updated
     * @param int user_id_log: the user ID of the person who updates the edited project
     * @returns int
     */
    function edit_project($project, $user_id_log)
    {
      $check=$this->check_params($project, $user_id_log);
      if (($project->project_id>0) && (strcmp($check,"") == 0))  {
        $db = new DB_Management;
        $db->open_db();
        $query ="UPDATE Project SET ";
        $query = $query."ProjectName='".$db->encode_string($project->project_name)."', ";
        // $query = $query."Order_Number=$project->order_number, ";
        $query = $query."HigherProject_ID=$project->higher_project_id, ";
        $query = $query.'Report_Flag='.($project->report_flag ? 1 : 0).', ';
        $query = $query."TargetWorkingTime='".$db->encode_string($project->target_working_time)."', ";
        $query = $query."TargetStartDate='".$db->encode_string($project->target_start_date)."', ";
        $query = $query."RealStartDate='".$db->encode_string($project->real_start_date)."', ";
        $query = $query."TargetEndDate='".$db->encode_string($project->target_end_date)."', ";
        $query = $query."RealEndDate='".$db->encode_string($project->real_end_date)."', ";
        $query = $query."Progress=".$db->sqlString($project->progress).", ";
        $query = $query."Status_ID=$project->status_id, ";
        $query = $query."Priority_ID=$project->priority_id, ";
        $query = $query."P_Partner='".$db->encode_string($project->p_partner)."', ";
        $query = $query."Description='".$db->encode_string($project->description)."', ";
        $query = $query."Cvs_Name='".$db->encode_string($project->cvs_name)."', ";
        $query = $query."Cvs_Branch='".$db->encode_string($project->cvs_branch)."', ";
        $query = $query."BookBackDate='".$db->encode_string($project->bookback_date)."', ";
        $query = $query."Remark='".$db->encode_string($project->remark)."', ";
        $query = $query."User_ID_Log=$user_id_log ";
        $query = $query."WHERE Project_ID=$project->project_id";
        $edit_result=$db->execute_query($query);
        $db->close_db();
        return $edit_result;
      } else {
          $error = new Error_Management;
          $error->error_page(0,$check);
          die();
      }
    } // end function edit_project
    
    
    /**
     * deletes a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project which in the database will be deleted
     * @returns int
     */
    function delete_project($project_id)
    {
      if ($project_id > 0 ) {
        $query ="DELETE FROM Project WHERE Project_ID=$project_id";
        $db = new DB_Management;
        $db->open_db();
        $delete_result = $db->execute_query($query);
        $db->close_db();
        return $delete_result;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    }  // end function delete_project


    /**
     * returns the Project object with the given project ID
     *
     * @param int project_id: project ID of the Project object which will be returned
     * @returns Project
     */
    function get_project($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();
        $proj = new Project;
        $query = "SELECT Proj.* FROM Project AS Proj WHERE "
               . " Proj.Project_ID = $project_id";
        $result = $db->execute_query($query);
        while ($row = $db->fetch_row($result)) {
          $proj->project_id= $row->Project_ID;
          $proj->project_name= $row->ProjectName;
          $proj->order_number= $row->Order_Number;
          $proj->higher_project_id = $row->HigherProject_ID;
          $proj->report_flag = $row->Report_Flag;
          $proj->target_working_time = $row->TargetWorkingTime;
          $proj->target_start_date = $row->TargetStartDate;
          $proj->real_start_date = $row->RealStartDate;
          $proj->target_end_date = $row->TargetEndDate;
          $proj->real_end_date = $row->RealEndDate;
          $proj->progress = $row->Progress;
          $proj->status_id = $row->Status_ID;
          $proj->priority_id = $row->Priority_ID;
          $proj->p_partner = $row->P_Partner;
          $proj->description = $row->Description;
          $proj->cvs_name = $row->Cvs_Name;
          $proj->cvs_branch = $row->Cvs_Branch;
          $proj->remark = $row->Remark;
          $proj->bookback_date = $row->BookBackDate;
        }
        $db->close_db();
        return $proj;
      }
      else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    }  // end function get_project


    /*
    function get_project2($project_id) {
      if ($project_id > 0) {
        if (!$this->proj_cache[$project_id]) {
          $db = new DB_Management;
          $db->open_db();

          $this->proj_cache[$project_id] = new Project;

          $query = "SELECT HigherProject_ID, ProjectName FROM Project WHERE Project_ID = '$project_id'";
          $result = $db->execute_query($query);

          $row = $db->fetch_row($result);
          $this->proj_cache[$project_id]->project_name = $row->ProjectName;
          $this->proj_cache[$project_id]->higher_project_id = $row->HigherProject_ID;

          // $this->proj_cache[$project_id] = $proj;

          $db->close_db();
        }
        return $this->proj_cache[$project_id];
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    }
    */
    

    function get_full_project_path($projid='') {
      if ($projid>0) {
        $pp = new Project;
        $proj = $pp->get_project($projid);
        if (strcmp($proj->higher_project_id,0)==0) {
          return $proj->project_name;
        } else {
          return $pp->get_full_project_path($proj->higher_project_id)."/".$proj->project_name;
        }
      } else {
        return "";
      }
    }

    function get_top_level_project_id($projid) {
      if ($projid>0) {
        $pp = new Project;
        $proj = $pp->get_project($projid);
        while (strcmp($proj->higher_project_id,0)!=0) {
          $proj = $pp->get_project($proj->higher_project_id);
        }
        return $proj->project_id;
      } else {
        return 0;
      }
    }
    
    /**
     * returns an array of Projects that fulfill the selection criteria.
     * If a parameter has a valid value, then in the query it will be considered.
     * If no parameters are given, all stored Projects are returned.
     *
     * @param int higher_project_id: project ID of the parent project
     * @param int order_number: order number of a project in a level of hierarchy of the projects
     * @param int status_id: status ID of a project
     * @param int priority_id: priority ID of a project
     * @param int not_status_id: status ID of a project, which not in the resultset may be
     * @param string begin_date: begin date of the period of time, in which one is queried,
     *  whether the date type is active in this period
     * @param string end_date: end date of the period of time, in which one is queried,
     *  whether the date type is active in this period
     * @param string date_type: determines whether the date parameters are scheduled or real
     * @param string orderby: to order the resultset by one or more attributes
     * @returns array
     */
    function get_projects($higher_project_id=-1, $order_number=-1, $status_id='', $priority_id='', $not_status_id='', $begin_date='', $end_date='', $date_type='', $orderby='')
    {
      $db = new DB_Management;
      $db->open_db();

      $pmc = new PM_Common;

      $projects = array();

      $query = "SELECT Proj.* FROM Project AS Proj, Status AS Stat, Priority AS Prio WHERE "
             . " Proj.Status_ID = Stat.Status_ID AND Prio.Priority_ID = Proj.Priority_ID ";

      // "0" is the project ID of the root of the projects
      if ($higher_project_id >= 0) {
        $query = $query . " AND Proj.HigherProject_ID= $higher_project_id";
      }
      // the first project in the order has the order number "0"
      if ($order_number >= 0) {
        $query = $query . " AND Proj.Order_Number= $order_number";
      }
      if ($status_id >0) {
        $query = $query . " AND Proj.Status_ID= $status_id";
      }
      if ($priority_id >0) {
        $query = $query . " AND Proj.Priority_ID= $priority_id";
      }
      if ($not_status_id >0) {
        $query = $query . " AND NOT (Proj.Status_ID= $not_status_id )";
      }
      if (($pmc->check_date($begin_date)) && ($pmc->check_date($end_date)) && ($this->check_date_type($date_type))) {
        $query = $query . " AND Proj.'$date_type' >= '$begin_date' AND Proj.'$date_type' <= '$end_date' ";
      }
      if (empty($orderby)) {
        $orderby="Project_ID";
      }
      $query = $query . " ORDER BY ". $orderby;

      $result = $db->execute_query($query);

      while ($row = $db->fetch_row($result)) {
        $foo = new Project;
        $foo->project_id= $row->Project_ID;
        $foo->project_name= $row->ProjectName;
        $foo->order_number= $row->Order_Number;
        $foo->higher_project_id = $row->HigherProject_ID;
        $foo->target_working_time = $row->TargetWorkingTime;
        $foo->target_start_date = $row->TargetStartDate;
        $foo->real_start_date = $row->RealStartDate;
        $foo->target_end_date = $row->TargetEndDate;
        $foo->real_end_date = $row->RealEndDate;
        $foo->progress = $row->Progress;
        $foo->status_id = $row->Status_ID;
        $foo->priority_id = $row->Priority_ID;
        $foo->p_partner = $row->P_Partner;
        $foo->description = $row->Description;
        $foo->cvs_name = $row->Cvs_Name;
        $foo->cvs_branch = $row->Cvs_Branch;
        $foo->remark = $row->Remark;
        array_push($projects,$foo);
      }

      $db->close_db();
      return $projects;
    } // end function get_projects


    /**
     * checks the parameter date_type which in the method get_projects is used.
     * if the datetype is correct then the method returns "1" else "0"
     *
     * @param string date_type: determines whether the date parameters are scheduled or real
     * @returns int
     */
    function check_date_type($date_type) {

      $check_result=0;
      if (strcmp($date_type,"TargetStartDate") == 0 ) {
        $check_result=1;
      } elseif (strcmp($date_type,"RealStartDate") == 0 ) {
        $check_result=1;
      } elseif (strcmp($date_type,"TargetEndDate") == 0 ) {
        $check_result=1;
      } elseif (strcmp($date_type,"RealEndDate") == 0 ) {
        $check_result=1;
      }
      return $check_result;
    }  // end function check_date_type
    
    
    /**
     * returns an array of project_ID�s of the projects which are active in the day in question
     *
     * @param string date: date of the day;
     *  whether it is the scheduled one or the real date depends on the parameter datetype
     * @param string date_type: determines whether the date parameter is scheduled or real
     * @returns array
     */
    function get_projects_on_day($date='0000-00-00', $date_type='')
    {
      $db = new DB_Management;
      $db->open_db();
      $query = "SELECT Proj.Project_ID FROM Project AS Proj, Status AS Stat, Priority AS Prio WHERE "
             . " Proj.Status_ID = Stat.Status_ID AND Prio.Priority_ID = Proj.Priority_ID ";

      if (strcasecmp($date_type,'real') == 0) {
        $query = $query . " AND (Proj.RealStartDate <= '$date') AND (Proj.RealStartDate <> '0000-00-00') AND ((Proj.RealEndDate >= '$date') OR (Proj.RealEndDate = '0000-00-00')) ";
      } else {
        $query = $query . " AND Proj.TargetStartDate <= '$date' AND Proj.TargetEndDate >= '$date' ";
      }
      $query = $query . " ORDER BY Project_ID";
      
      $result = $db->execute_query($query);
      $i=1;
      while ($row = $db->fetch_row($result)) {
        $projects_on_day[$i]= $row->Project_ID;
        $i++;
      }
      $db->close_db();
      return $projects_on_day;
    } // end function get_projects_on_day
    
    
    /**
     * returns either whether the project in question is active in the day in question or
     * the number of projects which are active in the day in question
     *
     * @param int project_id: project ID of the project in question
     * @param string date: date of the day;
     *  whether it is the scheduled one or the real date depends on the parameter datetype
     * @param string date_type: determines whether the date parameter is scheduled or real
     * @returns int
     */
     function check_project_on_day($project_id='', $date='0000-00-00', $date_type='')
     {
       $db = new DB_Management;
       $db->open_db();
       $query = "SELECT COUNT(Proj.Project_ID) FROM Project AS Proj, Status AS Stat, Priority AS Prio WHERE "
              . " Proj.Status_ID = Stat.Status_ID AND Prio.Priority_ID = Proj.Priority_ID ";

       if ($project_id > 0) {
         $query = $query . " AND Proj.Project_ID= $project_id";
       }
       if (strcasecmp($date_type,'real') == 0) {
         $query = $query . " AND (Proj.RealStartDate <= '$date') AND (Proj.RealStartDate <> '0000-00-00') AND ((Proj.RealEndDate >= '$date') OR (Proj.RealEndDate = '0000-00-00')) ";
       } else {
         $query = $query . " AND (Proj.TargetStartDate <= '$date') AND (Proj.TargetEndDate >= '$date') ";
       }

       $nr_of_projects_on_day = $db->get_single_value($query);
       $db->close_db();
       return $nr_of_projects_on_day;
     } // end function check_projects_on_day
    
    
     /**
     * returns whether the project in question is active in the month in question or not
     *
     * @param int project_id: project ID of the project in question
     * @param string range_day: number of days of the month in question
     * @param string range_month: the month in question
     * @param string range_year: the year of the month in question
     * @param string date_type: determines whether the date parameter is scheduled, real or both
     * @returns int
     */
     function check_project_in_range($project_id, $range_day, $range_month, $range_year, $date_type='')
     {
       $db = new DB_Management;
       $db->open_db();

       $begin_range=sprintf('%04d-%02d-%02d',$range_year,$range_month,1);
       $end_range=sprintf('%04d-%02d-%02d',$range_year,$range_month,$range_day);

       $query = "SELECT COUNT(Proj.Project_ID) FROM Project AS Proj WHERE Proj.Project_ID= $project_id";

      if (strcasecmp($date_type,'real') == 0) {
        $query = $query . " AND ((Proj.RealStartDate <= '$begin_range' AND Proj.RealEndDate >= '$begin_range') "
                        . " OR  ((Proj.RealStartDate <= '$end_range') AND (Proj.RealStartDate <> '0000-00-00') AND ((Proj.RealEndDate >= '$end_range') OR (Proj.RealEndDate = '0000-00-00'))) "
                        . " OR  (Proj.RealStartDate >= '$begin_range' AND Proj.RealEndDate <= '$end_range')) ";
      } elseif (strcasecmp($date_type,'target') == 0) {
        $query = $query . " AND ((Proj.TargetStartDate <= '$begin_range' AND Proj.TargetEndDate >= '$begin_range') "
                        . " OR  (Proj.TargetStartDate <= '$end_range' AND Proj.TargetEndDate >= '$end_range') "
                        . " OR  (Proj.TargetStartDate >= '$begin_range' AND Proj.TargetEndDate <= '$end_range')) ";
      } else {
        $query = $query . " AND ((Proj.RealStartDate <= '$begin_range' AND Proj.RealEndDate >= '$begin_range') "
                        . " OR  ((Proj.RealStartDate <= '$end_range') AND (Proj.RealStartDate <> '0000-00-00') AND ((Proj.RealEndDate >= '$end_range') OR (Proj.RealEndDate = '0000-00-00'))) "
                        . " OR  (Proj.RealStartDate >= '$begin_range' AND Proj.RealEndDate <= '$end_range') "
                        . " OR  (Proj.TargetStartDate <= '$begin_range' AND Proj.TargetEndDate >= '$begin_range') "
                        . " OR  (Proj.TargetStartDate <= '$end_range' AND Proj.TargetEndDate >= '$end_range') "
                        . " OR  (Proj.TargetStartDate >= '$begin_range' AND Proj.TargetEndDate <= '$end_range')) ";
      }
      
      $is_in_range = $db->get_single_value($query);
        
      $db->close_db();
      return $is_in_range;
    } // end function check_project_in_range


    /**
     * returns an array of project-IDs, project names and the depths of the projects in the projecttree
     * ordered in their hierarchy and order number; starts below the given root project
     *
     * @param int root_id: project ID of the root project
     * @param string active: parameter determines whether all or only the active projects are returned
     * @returns array
     */
    function get_project_tree($root_id, $active='')
    {
      if (!isset($db)) {
         $db = new DB_Management;
      }
      $db->open_db();

      $sub_project_id_array=array();
      $sub_project_name_array=array();
      $sub_project_depth_array=array();
      // indexparameter of the various arrays in the method make_project_tree
      $counter=1;
      // parameter for the depth of the project in the projecthierarchy relative to the depth of the given root project
      $depthscounter=0;
      $project_tree=$this->make_project_tree($root_id, $active, &$counter, &$depthscounter, &$sub_project_id_array, &$sub_project_name_array, &$sub_project_depth_array, $db);

      $db->close_db();
      return $project_tree;
    } // end function get_project_tree
    


    /**
     * returns an array of project-IDs, project names and the depths of the projects in the projecttree
     * ordered in their hierarchy and order number; starts below the given root project;
     * this method is called for so long recursively until all projects are passed
     * in the branch below the given root project
     *
     * @param int root_id: project ID of the root project
     * @param string active: parameter determines whether all or only the active projects are returned
     * @param int counter:  count the  number of projects, used as indexparameter of the various arrays
     * @param int depthscounter:  depth of the project in the projecthierarchy relative to the depth of the given root project
     * @param array sub_project_id:  array of project_IDs of the subprojects
     * @param array sub_project_name: array of project names of the subprojects
     * @param array $sub_project_depth: array of project depths of the subprojects
     * @param DB_Management db: connection to the database
     * @returns array
     * @private
     */
    function make_project_tree($root_id=0, $active='', &$counter,  &$depthscounter, &$sub_project_id, &$sub_project_name, &$sub_project_depth, $db)
    {
      $query = "SELECT Proj.Project_ID, Proj.ProjectName FROM Project AS Proj WHERE Proj.HigherProject_ID='$root_id' ";

	  switch($active) {
	  case 'A': case 'a':
        /* 5 means "inactive" */
        $query .= ' AND NOT (Proj.Status_ID = 5)';
        break;
	  case 'R': case 'r':
        /* 1 and 2 are "planning phase" and "in progress" ->  "running" */
        $query .= ' AND Proj.Status_ID IN (1,2) ';
        break;
      case 'F': case 'f':
        /* 4 means "finished" */
        $query .= ' AND Proj.Status_ID = 4';
        break;
      }
      $query = $query . " ORDER BY Proj.ProjectName";
      $result = $db->execute_query($query);

      while ($thread[$root_id] = $db->fetch_associative_array($result)) {
        $sub_project_id[$counter]= $thread[$root_id]['Project_ID'];
        $sub_project_name[$counter]= $thread[$root_id]['ProjectName'];
        $sub_project_depth[$counter]=$depthscounter;
        
        $counter++;
        $depthscounter++;
        // to generate the projecttree these method is recursive called
        $this->make_project_tree($thread[$root_id]['Project_ID'],$active, $counter, $depthscounter, $sub_project_id, $sub_project_name, $sub_project_depth, $db);
      }
      $depthscounter--;
       
      $sub_projects['project_id']=$sub_project_id;
      $sub_projects['project_name']=$sub_project_name;
      $sub_projects['project_depth']=$sub_project_depth;
      return $sub_projects;
    }  // end function make_project_tree
    
    
    function getProjectTree($active, &$roles) {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT Project_ID, ProjectName, HigherProject_ID FROM Project";

      if ((strcmp($active,'A') == 0) || (strcmp($active,'a') == 0)) {
      /* 5 means "inactive" */
        $query = $query . " WHERE (Status_ID <> 5)";
      }
      if ((strcmp($active,'R') == 0) || (strcmp($active,'r')==0)) {
      /* 1 and 2 are "planning phase" and "in progress" ->  "running" */
        $query = $query . " WHERE Status_ID IN (1,2)";
      }
      $query = $query . " ORDER BY ProjectName";

	  $projects = array();
      $result = $db->execute_query($query);
      while ($row = $db->fetch_associative_array($result)) {
      	$id = $row['Project_ID'];
      	$parent = $row['HigherProject_ID'];
      	$projects[$id]['name'] = $row['ProjectName'];
      	$projects[$parent]['childs'][] = $id;
      }
      $db->close_db();
      
      $project_id_list = array();
      $project_name_list = array();
      
      Project::makeProjectTree(&$projects, 0, "", &$project_id_list, &$project_name_list, &$roles);
      
	  return array($project_id_list, $project_name_list);

    }
    
    function makeProjectTree(&$projects, $root, $prefix, &$project_id_list, &$project_name_list, &$roles) {
    	if(isset($projects[$root]['childs'])) {
    		$mgr = $roles[$root][1];
	    	foreach($projects[$root]['childs'] as $id) {
    			$name = $prefix.$projects[$id]['name'];
    			if($mgr) {
    				$roles[$id][1] = 1;
    			}
    			$p_roles = $roles[$id];
    			if($p_roles) {
	    			$project_id_list[] = $id;
    				$project_name_list[] = $name;
    			}
    			Project::makeProjectTree(&$projects, $id, $name.'/', &$project_id_list, &$project_name_list, &$roles);
	    	}
    	}
    }
    
    function getProjects() {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT Project_ID, ProjectName, HigherProject_ID FROM Project ".
      					" WHERE (Status_ID <> 5) ORDER BY ProjectName";
	  $projects = array();
      $result = $db->execute_query($query);
      while ($row = $db->fetch_row($result)) {
      	$id = $row->Project_ID;
      	$projects[$id]->name = $row->ProjectName;
      	$projects[$row->HigherProject_ID]->childs[] = $id;
      }
      $db->close_db();
      return $projects;
    }
    
    function setRoles(&$projects, $userId) {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT Project_ID, ProjectRole_ID FROM Project_Assignment "
             			. "WHERE User_ID= $userId AND Status= 'A'";
      $result = $db->execute_query($query);
      while ($row = $db->fetch_row($result)) {
     	$projects[$row->Project_ID]->roles[$row->ProjectRole_ID] = 1;
      }

      $db->close_db();
     }
    
    /**
     * returns the project ID of the ancestor of the given project in the order of projects with the same root project
     *
     * @param int project_id: project ID of the project from which the ancestor is searched for
     * @returns int
     */
    function get_ancestor($project_id='')
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();
        
        $ancestor_id=0;

        $root_id=$this->get_higher_project_id($project_id);
        $order_number=$this->get_order_number($project_id);
        $has_parallel_project=$this->has_parallel_project($project_id);

        if ((!$has_parallel_project) && (($order_number == 1) || ($order_number == 0))) {
          $ancestor_id=0; // the order of projects with the same root project starts with "0"
        } else {
          if ((!($has_parallel_project)) && ($order_number>1)) {
            $order_number=$order_number-1;
          }

          $query = "SELECT Proj.Project_ID FROM Project AS Proj WHERE Proj.HigherProject_ID=$root_id AND Order_Number=$order_number ORDER BY Proj.Project_ID";
          $result = $db->execute_query($query);

          // a project can have one or more parallel projects with the same order number
          // in this case the project with the next lower project ID is the ancestor
          while ($row = $db->fetch_row($result))  {
            $ancestor_id=$row->Project_ID;
            if ($ancestor_id == $project_id) {
              $ancestor_id=$temp_id;
            }
            $temp_id=$ancestor_id;
          }
        }
        $db->close_db();
        return $ancestor_id;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    }  // end function get_ancestor
    
    
    /**
     * returns whether for the given project a project with the same order number and the same root project exists
     *
     * @param int project_id: project ID of the project from which the parallel project is searched for
     * @returns int
     */
    function has_parallel_project($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();
        
        $root_id=$this->get_higher_project_id($project_id);
        $order_number=$this->get_order_number($project_id);

        $query = "SELECT COUNT(Proj.Project_ID) FROM Project AS Proj WHERE Proj.HigherProject_ID=$root_id AND Order_Number=$order_number";
        $result = $db->get_single_value($query);
        if ($result > 1) {
          $parallel_project=1;
        } else {
          $parallel_project=0;
        }

        $db->close_db();
        return $parallel_project;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    }  //end function has_parallel_project
    
    
    /**
     * "inserts" a order number in the order of projects with the same root project and
     * returns the "new" order number of the project with the given ancestorproject
     *
     * @param int project_id: project ID of the ancestorproject of the new project
     * @param int root_id: project ID of the common parent project
     * @returns int
     */
    function add_order_number($ancestor_id, $root_id) {

      if ($ancestor_id > -1) {
        $db = new DB_Management;
        $db->open_db();

        $order_number=$this->get_order_number($ancestor_id);

        $query = "SELECT Proj.Project_ID, Proj.Order_Number FROM Project AS Proj WHERE Proj.HigherProject_ID=$root_id AND Order_Number>$order_number ORDER BY Proj.Order_Number, Proj.Project_ID";
        $result = $db->execute_query($query);

        // sets the new order number of the projects which are after the new project
        while ($row = $db->fetch_row($result)) {
          $project_id=$row->Project_ID;
          $new_order_number = 1 + $row->Order_Number;
          $this->set_order_number($project_id, $new_order_number);
        }
        // the new order number is the order number of the ancestorproject + 1
        $add_order_number=$order_number+1;

        $db->close_db();
        return $add_order_number;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } //end function add_order_number
    
    
    /**
     * "deletes" the order number of the given project
     *
     * @param int project_id: project ID of the project from which one the order number is deleted
     * @returns void
     */
    function delete_order_number($project_id)
    {
      if ($project_id > -1) {
        $db = new DB_Management;
        $db->open_db();
        
        if (!$this->has_parallel_project($project_id)) {
          $root_id=$this->get_higher_project_id($project_id);
          $order_number=$this->get_order_number($project_id);

          $query = "SELECT Proj.Project_ID, Proj.Order_Number FROM Project AS Proj WHERE Proj.HigherProject_ID=$root_id AND Order_Number>order_number ORDER BY Proj.Order_Number, Proj.Project_ID";
          $result = $db->execute_query($query);
          
          // sets the new order number of the projects which are after the deleted order number of the given project
          while ($row = $db->fetch_row($result)) {
            $project_id=$row->Project_ID;
            $new_order_number=($row->Order_Number)-1;
            $this->set_order_number($project_id, $new_order_number);
          }
        }

        $db->close_db();
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function delete_order_number
    

    /**
     * returns the name of the project
     *
     * @param int project_id: project ID of the project from which one the name is queried
     * @returns string
     */
    function get_project_name($project_id)
    {
      if ($project_id > 0) {
        // added a caching mechanism for project names
        //echo "foo - $project_id $this->pn_cache[$project_id] " . strcmp($this->pn_cache[$project_id],"") . " - foo\n";
	if (strcmp($this->pn_cache[$project_id],"")==0) {
	  $db = new DB_Management;
  	  $db->open_db();
          $query = "SELECT Proj.ProjectName From Project AS Proj WHERE Proj.Project_ID= $project_id";
          $this->pn_cache[$project_id] = $db->get_single_value($query);
          //echo "bar - $project_id $this->pn_cache[$project_id] " . strcmp($this->pn_cache[$project_id],"") . "- bar\n";
          $db->close_db();
        }
        $this->project_name = $this->pn_cache[$project_id];
        return $this->project_name;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_project_name
    
    
    /**
     * returns the order number of a project
     *
     * @param int project_id: project ID of the project from which one the order number is queried
     * @returns int
     */
    function get_order_number($project_id)
    {
      if ($project_id > -1) {
        if ($project_id == 0) {
          $this->order_number=0;
        } else {
          $db = new DB_Management;
          $db->open_db();
          $query = "SELECT Proj.Order_Number FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
          $this->order_number = $db->get_single_value($query);

          $db->close_db();
        }
        return $this->order_number;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_order_number
    
    
    /**
     * returns the project ID of the parent project of a project
     *
     * @param int project_id: project ID of the project from which one the parent project is queried
     * @returns int
     */
    function get_higher_project_id($project_id)
    {
      if ($project_id > -1) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.HigherProject_ID FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->higher_project_id = $db->get_single_value($query);

        $db->close_db();
        return $this->higher_project_id;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    }  // end function get_higher_project_id
    
   /**
    * returns all subprojects of a certain projects
    *
    * @param int project_id: project ID of the project from which the subprojects are queried
    * @return array of project IDs
    */
    function get_subprojects($project_id) {
      if ($project_id > -1) {
        $db = new DB_Management;
        $db->open_db();

        $id_array = array();

        $query = "SELECT Proj.Project_ID FROM Project AS Proj WHERE Proj.HigherProject_ID = $project_id";
        $result = $db->execute_query($query);
        while ($row = $db->fetch_row($result)) {
          array_push($id_array,$row->Project_ID);
        }
        $db->close_db();
        return $id_array;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    }


    /**
     * returns the target working time in manhours in order to be able to process the project successfully
     *
     * @param int project_id: project ID of the project from which one the target working time is queried
     * @returns real
     */
    function get_target_working_time($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.TargetWorkingTime FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->target_working_time = $db->get_single_value($query);

        $db->close_db();
        return $this->target_working_time;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_target_working_time
    
    
    /**
     * returns the target (scheduled) start date of a project
     *
     * @param int project_id: project ID of the project from which one the scheduled start date is queried
     * @returns string
     */
    function get_target_start_date($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.TargetStartDate FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->target_start_date = $db->get_single_value($query);

        $db->close_db();
        return $this->target_start_date;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_target_start_date
    
    /**
     * returns the real start date of a project
     *
     * @param int project_id: project ID of the project from which one the real start date is queried
     * @returns string
     */
    function get_real_start_date($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.RealStartDate FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->real_start_date = $db->get_single_value($query);

        $db->close_db();
        return $this->real_start_date;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_real_start_date
    
    
    /**
     * returns the target (scheduled) end date of a project
     *
     * @param int project_id: project ID of the project from which one the scheduled end date is queried
     * @returns string
     */
    function get_target_end_date($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.TargetEndDate FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->target_end_date = $db->get_single_value($query);

        $db->close_db();
        return $this->target_end_date;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // function get_target_end_date
    
    
    /**
     * returns the real end date of a project
     *
     * @param int project_id: project ID of the project from which one the real end date is queried
     * @returns string
     */
    function get_real_end_date($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.RealEndDate FROM Project AS Proj WHERE Proj.Project_ID= $project_id";
        $this->real_end_date = $db->get_single_value($query);

        $db->close_db();
        return $this->real_end_date;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // function get_real_end_date


    /**
     * returns the estimated progress in percent of a project
     *
     * @param int project_id: project ID of the project from which one the progress is queried
     * @returns int
     */
    function get_progress($project_id) {

      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.Progress FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->progress = $db->get_single_value($query);

        $db->close_db();
        return $this->progress;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_progress
    
    
    /**
     * returns the status ID of a project
     *
     * @param int project_id: project ID of the project from which one the statud ID is queried
     * @returns int
     * @see class.status.inc
     */
    function get_status_id($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.Status_ID FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->status_id = $db->get_single_value($query);

        $db->close_db();
        return $this->status_id;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_status_id


    /**
     * returns the priority ID of a project
     *
     * @param int project_id: project ID of the project from which one the priority ID is queried
     * @returns int
     * @see class.priority.inc
     */
    function get_priority_id($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.Priority_ID FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->priority_id = $db->get_single_value($query);

        $db->close_db();
        return $this->priority_id;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_priority_id

     
    /**
     * returns the project partner(s) of a project
     *
     * @param int project_id: project ID of the project from which one the project partners are queried
     * @returns string
     */
    function get_p_partner($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.P_Partner FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->p_partner = $db->get_single_value($query);

        $db->close_db();
        return $this->p_partner;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_p_partner


    /**
     * returns the description of a project
     *
     * @param int project_id: project ID of the project from which one the description is queried
     * @returns string
     */
     function get_description($project_id)
     {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.Description FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->description = $db->get_single_value($query);

        $db->close_db();
        return $this->description;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    }
    

    /**
     * returns the name of the project in the Concurrent Versions System (CVS)
     *
     * @param int project_id: project ID of the project from which one the name in the CVS is queried
     * @returns string
     */
    function get_cvs_name($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.Cvs_Name FROM Project AS Proj WHERE Proj.Project_ID = $project_id";
        $this->cvs_name = $db->get_single_value($query);

        $db->close_db();
        return $this->cvs_name;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_cvs_name
    
    
    /**
     * returns a branch of the project in the Concurrent Versions System (CVS)
     *
     * @param int project_id: project ID of the project from which one a branch in the CVS is queried
     * @returns string
     */
    function get_cvs_branch($project_id)
    {

      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.Cvs_Branch FROM Project AS Proj WHERE Proj.Project_ID= $project_id";
        $this->cvs_branch = $db->get_single_value($query);

        $db->close_db();
        return $this->cvs_branch;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_cvs_branch
    
    
    /**
     * returns the remarkable details of the project
     *
     * @param int project_id: project ID of the project from which one the remarks are queried
     * @returns string
     */
    function get_remark($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT Proj.Remark FROM Project AS Proj WHERE Proj.Project_ID= $project_id";
        $this->remark = $db->get_single_value($query);

        $db->close_db();
        return $this->remark;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_remark
    
	function name_is_unique()
	{
		$id = isset($this->project_id) ? $this->project_id : -1;
        $db = new DB_Management;
        $db->open_db();
        $stmt = $db->connection->prepare('select count(*) from Project where HigherProject_ID=? and ProjectName=? and Project_ID<>?');
        $res = &$db->connection->execute($stmt, array($this->higher_project_id, $this->project_name, $id));
        $row = &$res->fetchRow();
        $count = $row[0];
        $db->close_db();
        return $count==0;
	}

    /**
     * checks the validity of the attribute values of a Project object and
     * if the validity is ok then returns "" else returns the corresponding check message
     *
     * @param Project project: the Project object which has to be checked
     * @param int user_id_log: the user ID of the person which has to be checked
     * @returns string
     */
    function check_params($project, $user_id_log)
    {
	  if(! $project->name_is_unique()) {
	  	return i18n("Es existiert bereits ein Teilprojekt mit diesem Namen!");
	  }

      $pm_common = new PM_Common;
      $db = new DB_Management;
      $db->open_db();

      $check_message = i18n("F&uuml;r den/die Parameter ");
      $check_counter = 0;

      if (strcmp($project->project_name,"") == 0) {
        $check_message = $check_message . i18n("Projektname");
        $check_counter++;
      }
      if ($project->order_number < 0) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Reihenfolgenummer");
        $check_counter++;
      }
      if ($db->get_count("Project", "Project_ID = $project->higher_project_id") != 1) {
        if ($project->higher_project_id != 0) {
          if ($check_counter>0){
            $check_message = $check_message . ", ";
          }
          $check_message = $check_message . i18n("&Uuml;bergeordnetes Projekt");
          $check_counter++;
        }
      }
      if ($project->target_working_time < 0) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Sollaufwand");
        $check_counter++;
      }
      if (!($pm_common->check_date($project->target_start_date))) {
         if ($check_counter>0){
           $check_message = $check_message . ", ";
         }
         $check_message = $check_message . i18n("Geplanter Beginntermin");
         $check_counter++;
      }
      if ((!(strcmp($project->real_start_date,"0000-00-00")== 0)) && (!empty($project->real_start_date))) {
        if (!($pm_common->check_date($project->real_start_date))) {
           if ($check_counter>0){
             $check_message = $check_message . ", ";
           }
           $check_message = $check_message . i18n("Tats&auml;chlicher Beginntermin");
           $check_counter++;
        } else {
          $real_start_is_set=1;
        }
      }
      if (!($pm_common->check_date($project->target_end_date))) {
         if ($check_counter>0){
           $check_message = $check_message . ", ";
         }
         $check_message = $check_message . i18n("Geplanter Endtermin");
         $check_counter++;
      }
      if ((!(strcmp($project->real_end_date,"0000-00-00")== 0)) && (!empty($project->real_end_date))) {
        if (!($pm_common->check_date($project->real_end_date))) {
           if ($check_counter>0){
             $check_message = $check_message . ", ";
           }
           $check_message = $check_message . i18n("Tats&auml;chlicher Endtermin");
           $check_counter++;
        } elseif (($real_start_is_set)&&($pm_common->get_date_difference($project->real_start_date, $project->real_end_date)<0)) {
          if ($check_counter>0){
            $check_message = $check_message . ", ";
          }
          $check_message = $check_message . i18n("Differenz Tats&auml;chlicher Beginntermin/Tats&auml;chlicher Endtermin ");
          $check_counter++;
        } elseif (!($real_start_is_set)) {
          if ($check_counter>0){
            $check_message = $check_message . ", ";
          }
          $check_message = $check_message . i18n("Tats&auml;chlicher Beginntermin");
          $check_counter++;
        }
      }
      if ($pm_common->get_date_difference($project->target_start_date, $project->target_end_date)<0) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Differenz Geplanter Beginntermin/Geplanter Endtermin ");
        $check_counter++;
      }
      if (($project->progress < 0) || ($project->progress > 100)) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Gesch&auml;tzter Fortschritt");
        $check_counter++;
      }
      if (($project->status_id < 1) || ($db->get_count("Status", "Status_ID = $project->status_id") != 1)) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Status-ID");
        $check_counter++;
      }
      if (($project->priority_id < 1) || ($db->get_count("Priority", "Priority_ID = $project->priority_id") != 1)) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Priorit&auml;t-ID");
        $check_counter++;
      }
      if ($project->bookback_date != '0000-00-00' && !($pm_common->check_date($project->bookback_date))) {
           if ($check_counter>0){
             $check_message = $check_message . ", ";
           }
           $check_message = $check_message . i18n("zur&uuml;ckbuchen bis");
           $check_counter++;
      }
      if ($user_id_log < 1) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Bearbeiter-Benutzer-ID");
        $check_counter++;
      }

      $check_message = $check_message . i18n(" wurden keine oder falsche Werte eingegeben! &Uuml;berpr&uuml;fen Sie ihre Eingabe nochmal!");

      if ($check_counter == 0) {
        $check_message="";
      }
      
      $db->close_db();
      return $check_message;
    } // end function check_params
    
    
    /**
     * sets the project name of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the name is set
     * @param string project_name: the project name which is set
     * @returns int
     */
    function set_project_name($project_id, $project_name)
    {
      if (($project_id > 0) && (!empty($project_name))){
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET ProjectName=$project_name WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        /* update caches */
        /*
        if ($this->proj_cache[$project_id]!=null) {
          $this->proj_cache[$project_id]->project_name = $project_name;
        }
        */
        $this->pn_cache[$project_id] = $this->project_name = $project_name;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_project_name
    
    
    /**
     * sets the order number of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the order number is set
     * @param int order_number: the order number which is set
     * @returns int
     */
    function set_order_number($project_id, $order_number)
    {
      if (($project_id > 0) && ($order_number > -1)){
        $db = new DB_Management;
        $db->open_db();
        if ($order_number==0) {
          $order_number=1;
        }
        $query ="UPDATE Project SET Order_Number=$order_number WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->order_number = $order_number;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end set_order_number
    
    
    /**
     * sets the project ID of the parent project of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the parent project ID is set
     * @param int higher_project_id: the the project ID of the parent project which is set
     * @returns int
     */
    function set_higher_project_id($project_id, $higher_project_id)
    {
      if (($project_id > 0) && ($higher_project_id >= 0)){
        $db = new DB_Management;
        $db->open_db();
        if ($db->get_count("Project", "Project_ID = $higher_project_id") == 1){
          $query ="UPDATE Project SET HigherProject_ID=$higher_project_id WHERE Project_ID=$project_id";
          $update_result=$db->execute_query($query);
          $db->close_db();
          $this->higher_project_id = $higher_project_id;
          return $update_result;
        } else {
          $db->close_db();
          $error = new Error_Management;
          $error->error_page(20,'');
          die();
        }
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_higher_project_id
    

    /**
     * sets the target working time in manhours of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the target working time is set
     * @param real target_working_time: the target working time which is set
     * @returns int
     */
    function set_target_working_time($project_id, $target_working_time)
    {
      if (($project_id > 0) && ($this->check_time($target_working_time))){
        $db = new DB_Management;
        $db->open_db();
        $query ="UPDATE Project SET TargetWorkingTime='".$db->encode_string($target_working_time)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->target_working_time = $target_working_time;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_target_working_time

     
    /**
     * sets the target (scheduled) start date of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the target start date is set
     * @param string target_start_date: the target start date which is set
     * @returns int
     */
    function set_target_start_date($project_id, $target_start_date)
    {
      $pmc = new PM_Common;
      if (($project_id > 0) && ($pmc->check_date($target_start_date))){
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET TargetStartDate='".$db->encode_string($target_start_date)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->target_start_date = $target_start_date;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_target_start_date
    
    
    /**
     * sets the real start date of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the real start date is set
     * @param string real_start_date: the real start date which is set
     * @returns int
     */
    function set_real_start_date($project_id, $real_start_date)
    {
      $pmc = new PM_Common;
      if (($project_id > 0) && ($pmc->check_date($real_start_date))){
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET RealStartDate='".$db->encode_string($real_start_date)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->real_start_date = $real_start_date;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_real_start_date
    
    
    /**
     * sets the target (scheduled) end date of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the target end date is set
     * @param string target_end_date: the target end date which is set
     * @returns int
     */
    function set_target_end_date($project_id, $target_end_date) {

      $pmc = new PM_Common;
      if (($project_id > 0) && ($pmc->check_date($target_end_date))){
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET TargetEndDate='".$db->encode_string($target_end_date)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->target_end_date = $target_end_date;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_target_end_date


    /**
     * sets the the real end date of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the real end date is set
     * @param string real_end_date: the real end date which is set
     * @returns int
     */
    function set_real_end_date($project_id, $real_end_date)
    {
      $pmc = new PM_Common;
      if (($project_id > 0) && ($pmc->check_date($real_end_date))){
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET RealEndDate='".$db->encode_string($real_end_date)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->real_end_date =$real_end_date;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_real_end_date
    
    

    /**
     * sets the estimated progress in percent of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the progress is set
     * @param int progress: the estimated progress which is set
     * @returns int
     */
    function set_progress($project_id, $progress)
    {
      if (($project_id > 0) && (!empty($progress))){
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET Progress=$progress WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->progress = $progress;
        return $update_result;
      } else {
          $error = new Error_Management;
          $error->error_page(2,'');
          die();
      }
    } // end function set_progress
    
    
    /**
     * sets the status ID of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the status ID is set
     * @param int status_id: the status ID which is set
     * @param int user_id_log: the user ID of the user who sets the new status ID entry
     * @returns int
     * @see class.status.inc
     */
    function set_status_id($project_id, $status_id, $user_id_log)
    {
      if (($project_id > 0) && ($status_id > 0)){
        $db = new DB_Management;
        $db->open_db();
        if (!($db->get_count("User", "User_ID=$user_id_log") == 1)) {
          $db->close_db();
          $error = new Error_Management;
          $error->error_page(30,'');
          die();
        }
        if ($db->get_count("Status", "Status_ID = $status_id") == 1){
          $query ="UPDATE Project SET Status_ID=$status_id WHERE Project_ID=$project_id";
          $update_result=$db->execute_query($query);
          $db->close_db();
          $this->status_id = $status_id;
          return $update_result;
        } else {
          $db->close_db();
          $error = new Error_Management;
          $error->error_page(150,'');
          die();
        }
      } else {
          $error = new Error_Management;
          $error->error_page(2,'');
          die();
      }
    } // function set_status_id
    
    
    /**
     * sets the priority ID of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the priority ID is set
     * @param int priority_id: the priority ID which is set
     * @returns int
     * @see class.priority.inc
     */
    function set_priority_id($project_id, $priority_id)
    {
      if (($project_id > 0) && ($priority_id > 0)){
        $db = new DB_Management;
        $db->open_db();
        if ($db->get_count("Priority", "Priority_ID = $priority_id") == 1){
          $query ="UPDATE Project SET Priority_ID=$priority_id WHERE Project_ID=$project_id";
          $update_result=$db->execute_query($query);
          $db->close_db();
          $this->priority_id= $priority_id;
          return $update_result;
        } else {
          $db->close_db();
          $error = new Error_Management;
          $error->error_page(140,'');
          die();
        }
      } else {
          $error = new Error_Management;
          $error->error_page(2,'');
          die();
      }
    } // end function set_priority_id
    
    
    /**
     * sets the project partner(s) of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the project partners are set
     * @param string p_partner: the project partners which is set
     * @returns int
     */
    function set_p_partner($project_id, $p_partner)
    {
      if (($project_id > 0) && (!empty($p_partner))){
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET P_Partner='".$db->encode_string($p_partner)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->p_partner = $p_partner;
        return $update_result;
      } else {
          $error = new Error_Management;
          $error->error_page(2,'');
          die();
      }
    } // end function set_p_partner
    
    
    /**
     * sets the description of a project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the description is set
     * @param string description: the description which is set
     * @returns int
     */
    function set_description($project_id, $description)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET Description='".$db->encode_string($description)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->description = $description;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_description
    
    
    /**
     * sets the name of the project in the Concurrent Versions System (CVS) and
     * returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the name in the CVS is set
     * @param string cvs_name: the name of the project in the Concurrent Versions System which is set
     * @returns int
     */
    function set_cvs_name($project_id, $cvs_name)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET Cvs_Name='".$db->encode_string($cvs_name)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->cvs_name= $cvs_name;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_cvs_name
    
    
    /**
     * sets the branch of the project in the Concurrent Versions System (CVS) and
     * returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the branch in the CVS is set
     * @param string cvs_branch: the branch of the project in the Concurrent Versions System which is set
     * @returns int
     */
    function set_cvs_branch($project_id, $cvs_branch)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET Cvs_Branch='".$db->encode_string($cvs_branch)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->cvs_branch = $cvs_branch;
        return $update_result;
      } else {
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_cvs_branch


    /**
     * sets remarkable details of the project and returns "1" on success or "0" on error
     *
     * @param int project_id: project ID of the project from which one the reamarks are set
     * @param string remark: remarks which are set
     * @returns int
     */
    function set_remark($project_id, $remark)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project SET Remark='".$db->encode_string($remark)."' WHERE Project_ID=$project_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->remark =$remark;
        return $update_result;
      } else {
          $error = new Error_Management;
          $error->error_page(2,'');
          die();
      }
    } // end function set_remark

  }   // end Class Project
?>
