<?php

  // include class PM_Common for common used methods
  include_once($offset."class.pm_common.inc");
  // include class Project for the method get_project_tree
  include_once($offset."class.project.inc");
 

  /**
   * The class Project_Assignment is used for the administration of the information appertaining
   * an user assignment to a project.
   * Also methods for various querys are available.
   *
   * @author Martin Jobst (JMF)
   * @version 1.0 beta 2002/02/12
   */
  class Project_Assignment
  {
    /**
     * unique project assignment identification number (ID) which is generated by the database
     * @type int
     */
    var $p_assignment_id;
    /**
     * project ID of the project assignment
     * @type int
     * @see class.project.inc
     */
    var $project_id;
    /**
     * user ID of the project assignment
     * @type int
     * @see class.user.inc
     */
    var $user_id;
    /**
     * project role ID of the project assignment
     * @type int
     * @see class.project_role.inc
     */
    var $project_role_id;
    /**
     * status of the project assignment, "A"ctiv or "N"ot active
     * @type string
     */
    var $status;
    /**
     * description of the project assignment
     * @type string
     */
    var $description;

    var $prid_cache;

    var $pid_cache;


    /**
     * simple constructor to generate a Project_Assignment object
     * @returns void
     */
    /* function Project_Assignment(){ } */
    
    
    /**
     * enhanced constructor to generate a Project_Assignment object
     *
     * @param int p_assignment_id: project assignment ID of the project assignment which will be generated
     * @returns void
     */
    function Project_Assignment($p_assignment_id='')
    {
      $this->p_assignment_id=$p_assignment_id;
      $this->prid_cache = array();
      $this->pid_cache = array();
    }


    /**
     * creates a new project assignment and returns the ID of the new project assignment
     *
     * @param Project_Assignment p_assignment: the new project assignment that in the database will be stored
     * @param int user_id_log: the user ID of the person who creates the new project assignment entry
     * @returns int
     */
    function new_p_assignment($p_assignment,$user_id_log)
    {
      $check=$this->check_params($p_assignment,$user_id_log);
      if (strcmp($check,"") == 0)  {
        $db = new DB_Management;
        $db->open_db();
        $query ="INSERT INTO Project_Assignment (Project_ID, User_ID, ProjectRole_ID, Status, Description, User_ID_Log) VALUES (";
        $query = $query."$p_assignment->project_id, ";
        $query = $query."$p_assignment->user_id, ";
        $query = $query."$p_assignment->project_role_id, ";
        $query = $query."'$p_assignment->status', ";
        $query = $query."'".$db->encode_string($p_assignment->description)."', ";
        $query = $query."$user_id_log )";

        $db->execute_query($query);

        if ($db->get_affected_rows() > 0 ) {
          $new_p_assignment_id = $db->get_insert_id('Project_Assignment','P_Assignment_ID');
        } else {
          $new_p_assignment_id=-1;
        }
        $db->close_db();
        return  $new_p_assignment_id;
      } else {
        $error = new Error_Management;
        $error->error_page(0,$check);
        die();
      }
    } // end function new_p_assignment


    /**
     * updates the edited attributes of a project assignment and returns "1" on success or "0" on error
     *
     * @param Project_Assignment p_assignment: the edited project assignment that in the database will be updated
     * @param int user_id_log: the user ID of the person who updates the edited project assignment
     * @returns int
     */
    function edit_p_assignment($p_assignment, $user_id_log)
    {
      $check=$this->check_params($p_assignment, $user_id_log);
      if (($p_assignment->p_assignment_id>0) && (strcmp($check,"") == 0))  {
        $db = new DB_Management;
        $db->open_db();
        $query ="UPDATE Project_Assignment SET ";
        $query = $query."Project_ID=$p_assignment->project_id, ";
        $query = $query."User_ID=$p_assignment->user_id, ";
        $query = $query."ProjectRole_ID=$p_assignment->project_role_id, ";
        $query = $query."Status='$p_assignment->status', ";
        $query = $query."Description='".$db->encode_string($p_assignment->description)."', ";
        $query = $query."User_ID_Log=$user_id_log ";
        $query = $query."WHERE P_Assignment_ID=$p_assignment->p_assignment_id";

        $edit_result=$db->execute_query($query);
        $db->close_db();
        return $edit_result;
      } else {
        $error = new Error_Management;
        $error->error_page(0,$check);
        die();
      }
    } // end function edit_p_assignment


    /**
     * deletes a project assignment and returns "1" on success or "0" on error
     *
     * @param int p_assignment_id: project assignment ID of the project assignment which in the database will be deleted
     * @returns int
     */
    function delete_p_assignment($p_assignment_id)
    {
      if ($p_assignment_id > 0 ) {
        $query ="DELETE FROM Project_Assignment WHERE P_Assignment_ID=$p_assignment_id";
        $db = new DB_Management;
        $db->open_db();
        $delete_result = $db->execute_query($query);
        $db->close_db();
        return $delete_result;
      } else {
        $error = new Error_Management;
        $error->error_page(160,'');
        die();
      }
    } // end function delete_p_assignment


    /**
     * returns the Project_Assignment object with the given project assignment ID
     *
     * @param int p_assignment_id: project assignment ID of the Project_Assignment object which will be returned
     * @returns Project_Assignment
     */
    function get_p_assignment($p_assignment_id)
    {
      if ($p_assignment_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT PrAs.* FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
             . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
             . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.P_Assignment_ID= $p_assignment_id";
        $result = $db->execute_query($query);

        while ($row = $db->fetch_row($result)) {
          $this->p_assignment_id= $row->P_Assignment_ID;
          $this->project_id = $row->Project_ID;
          $this->user_id = $row->User_ID;
          $this->project_role_id= $row->ProjectRole_ID;
          $this->status = $row->Status;
          $this->description = $row->Description;
        }
        $db->close_db();
        return $this;
      } else {
        $error = new Error_Management;
        $error->error_page(160,'');
        die();
      }
    } // end function get_p_assignment
    
    
    /**
     * returns an array of Project_Assignment objects that fulfill the selection criteria.
     * If a parameter has a valid value, then in the query it will be considered.
     * If no parameters are given, all stored Project_Assignments are returned.
     *
     * @param int user_id: user ID of the project assignment
     * @param int project_id: project ID of the project assignment
     * @param int project_role_id: project role ID of the project assignment
     * @param string status: determines whether the project assignment which are queried for are "a"ctive or "n"ot active
     * @param string orderby: to order the resultset by one or more attributes
     * @returns array
     */
    function get_p_assignments($user_id='', $project_id='', $project_role_id='', $status='', $orderby='')
    {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT PrAs.* FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
             . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
             . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID";

      if ($user_id > 0) {
        $query = $query . " AND PrAs.User_ID= $user_id";
      }
      if ($project_id > 0) {
        $query = $query . " AND PrAs.Project_ID= $project_id";
      }
      if ($project_role_id > 0) {
        $query = $query . " AND PrAs.ProjectRole_ID= $project_role_id";
      }
      if ((strcasecmp($status,'A') == 0) || (strcasecmp($status,'N') == 0)) {
       $query = $query . " AND PrAs.Status= '$status' ";
      }
      if (empty($orderby)) {
        $orderby="P_Assignment_ID";
      }
      $query = $query . " ORDER BY ". $orderby;

      $result = $db->execute_query($query);

      $i=1;
      while ($row = $db->fetch_row($result)) {
        $this->p_assignment_id= $row->P_Assignment_ID;
        $this->project_id = $row->Project_ID;
        $this->user_id = $row->User_ID;
        $this->project_role_id= $row->ProjectRole_ID;
        $this->status = $row->Status;
        $this->description = $row->Description;
        $p_assignments[$i]=$this;
        $i++;
      }

      $db->close_db();
      return $p_assignments;
    } // end function get_p_assignments
    
    function get_p_assignment_roles($user_id='', $project_id='', $project_role_id='', $status='', $orderby='')
    {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT PrAs.* FROM Project_Assignment AS PrAs"
             . " WHERE 0=0";
      if ($user_id > 0) {
        $query = $query . " AND PrAs.User_ID= $user_id";
      }
      if ($project_id > 0) {
        $query = $query . " AND PrAs.Project_ID= $project_id";
      }
      if ($project_role_id > 0) {
        $query = $query . " AND PrAs.ProjectRole_ID= $project_role_id";
      }
      if ((strcasecmp($status,'A') == 0) || (strcasecmp($status,'N') == 0)) {
       $query = $query . " AND PrAs.Status= '$status' ";
      }
      if (! empty($orderby)) {
	   $query = $query . " ORDER BY ". $orderby;
      }

      $result = $db->execute_query($query);
      while ($row = $db->fetch_row($result)) {
     	$roles[$row->Project_ID][$row->ProjectRole_ID] = 1;
      }

      $db->close_db();
      return $roles;
    } // end function get_p_assignment_roles
    
    
    /**
     * returns the number of project assignments that fulfill the selection criteria.
     * If a parameter has a valid value, then in the query it will be considered.
     *
     * @param int project_id: project ID of the project assignment
     * @param int user_id: user ID of the project assignment
     * @param int project_role_id: project role ID of the project assignment
     * @param string status: determines whether the project assignment which are queried for are "a"ctive or "n"ot active
     * @returns int
     */
    function exist_p_assignment($project_id, $user_id, $project_role_id, $status)
    {
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT COUNT(PrAs.P_Assignment_ID) FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
             . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
             . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.User_ID= $user_id AND PrAs.Project_ID= $project_id";

      if ($project_role_id > 0) {
        $query = $query . " AND PrAs.ProjectRole_ID= $project_role_id";
      }
      if ((strcasecmp($status,'A') == 0) || (strcasecmp($status,'N') == 0)) {
       $query = $query . " AND PrAs.Status= '$status' ";
      }

      $exist_p_assignment = $db->get_single_value($query);

      $db->close_db();
      return $exist_p_assignment;
    } // end function exist_p_assignment
    
    
    /**
     * returns an array of project IDs of the projects at which the user is project manager or
     * the user is project manager of a parent project
     *
     * @param int user_id: user ID of the project assignment
     * @returns array
     */
    function get_projects_user_is_pm($user_id)
    {
      $project= new Project;
      $db = new DB_Management;
      $db->open_db();

      $query = "SELECT PrAs.Project_ID FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
             . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
             . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.ProjectRole_ID=1 AND PrAs.Status='A'"
             . " AND PrAs.User_ID=$user_id ORDER BY Project_ID";

      $result = $db->execute_query($query);

      $ctr=1;
      $pm_project_id_list=array();
      while ($row = $db->fetch_row($result)) {
        $root_project_id= $row->Project_ID;
        if (!(in_array($root_project_id,$pm_project_id_list))) {
          // gets all projects in the projecttree; starts below the given root project
          $project_tree_array=$project->get_project_tree($root_project_id,'A');
          $project_id_array=$project_tree_array['project_id'];
          $project_id_array[0]=$root_project_id;
          for($j=0; $j < count($project_id_array); $j++) {
            $check_project_id=$project_id_array[$j];
            // checks whether the project already in the array of the projects is, at which the user is project manager
            if (!(in_array($check_project_id,$pm_project_id_list))) {
              // if the project is already not in the array then it will be added
              $pm_project_id_list[$ctr]=$project_id_array[$j];
              $ctr++;
            }
          }
        }
      }
      $db->close_db();
      return $pm_project_id_list;
    } // end function get_projects_user_is_pm

    /**
     * returns whether an user is the project manager of the given project
     * or of a parent project of the given project
     *
     * @param int user_id: user ID of the project assignment
     * @param int project_id: project ID of the project assignment
     * @returns array
     */
    function user_is_higher_pm($user_id, $project_id)
    {
      $projects_user_is_pm_array=$this->get_projects_user_is_pm($user_id);
      if ((in_array($project_id,$projects_user_is_pm_array))) {
        $user_is_higher_pm=true;
      } else {
        $user_is_higher_pm=false;
      }
      return $user_is_higher_pm;
    } // function user_is_higher_pm
    
    
    /**
     * returns the user ID of the project manager of the given project
     *
     * @param int project_id: project ID of the project assignment from which one the project manager is queried
     * @returns string
     */
    function get_project_manager($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT PrAs.User_ID FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
               . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
               . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.ProjectRole_ID= 1 AND"
               . " PrAs.Status= 'A' AND PrAs.Project_ID= $project_id";

        $p_manager = $db->get_single_value($query);
        $db->close_db();
        return $p_manager;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_project_manager


    /**
     * returns the project assignment ID of the project assignment from which one
     * the user is the project manager of the given project
     *
     * @param int project_id: project ID of the project from which one the project manager is queried
     * @returns string
     */
    function get_pm_assignment_id($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();
        
        $user_id=$this->get_project_manager($project_id);
        if (empty($user_id)) {
           $user_id=0;
        }

        $query = "SELECT PrAs.P_Assignment_ID FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
               . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
               . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.ProjectRole_ID= 1 AND"
               . " PrAs.Status= 'A' AND PrAs.Project_ID= $project_id AND PrAs.User_ID=$user_id";

        $pm_assignment_id = $db->get_single_value($query);
        $db->close_db();
        return $pm_assignment_id;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_pm_assignment_id
    
    
    /**
     * returns the user ID of the quality assurance manager of the given project
     *
     * @param int project_id: project ID of the project assignment from which one the quality assurance manager is queried
     * @returns string
     */
    function get_qa_manager($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();
        
        $query = "SELECT PrAs.User_ID FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
               . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
               . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.ProjectRole_ID= 2 AND"
               . " PrAs.Status = 'A' AND PrAs.Project_ID= $project_id";

        $qa_manager = $db->get_single_value($query);
        $db->close_db();
        return $qa_manager;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_qa_manager
    
    
    /**
     * returns the project assignment ID of the project assignment from which one
     * the user is the quality assurance manager of the given project
     *
     * @param int project_id: project ID of the project from which one the quality assurance manager is queried
     * @returns string
     */
    function get_qm_assignment_id($project_id)
    {
      if ($project_id > 0) {
        $db = new DB_Management;
        $db->open_db();
        
        $user_id=$this->get_qa_manager($project_id);

        if (empty($user_id)) {
          $user_id = 0;
        }

        $query = "SELECT PrAs.P_Assignment_ID FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
               . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
               . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.ProjectRole_ID= 2 AND"
               . " PrAs.Status = 'A' AND PrAs.Project_ID= $project_id AND PrAs.User_ID='$user_id'";

        $qm_assignment_id = $db->get_single_value($query);
        $db->close_db();
        return $qm_assignment_id;
      } else {
        $error = new Error_Management;
        $error->error_page(20,'');
        die();
      }
    } // end function get_qm_assignment_id
    
    
    /**
     * returns the project ID of the project assignment
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the project ID is queried
     * @returns int
     * @see class.project.inc
     */
    function get_project_id($p_assignment_id)
    {
      if ($p_assignment_id > 0) {
        if (strcmp($this->pid_cache[$p_assignment_id],"")==0) {
          $db = new DB_Management;
          $db->open_db();

        
          $query = "SELECT PrAs.Project_ID FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
                 . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
                 . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.P_Assignment_ID= $p_assignment_id";
          $this->pid_cache[$p_assignment_id] = $db->get_single_value($query);

          $db->close_db();
        }
        $this->project_id = $this->pid_cache[$p_assignment_id];
        return $this->project_id;
      } else {
        $error = new Error_Management;
        $error->error_page(160,'');
        die();
      }
    } // function get_project_id
    
    
    /**
     * returns the user ID of the project assignment
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the user ID is queried
     * @returns int
     * @see class.user.inc
     */
    function get_user_id($p_assignment_id)
    {
      if ($p_assignment_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT PrAs.User_ID FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
               . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
               . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.P_Assignment_ID= $p_assignment_id";
        $this->user_id = $db->get_single_value($query);

        $db->close_db();
        return $this->user_id;
      } else {
        $error = new Error_Management;
        $error->error_page(160,'');
        die();
      }
    } // end function get_user_id


    /**
     * returns an array of user IDs of all users which are assigned to the given project
     *
     * @param int project_id: project ID of the project from which one the user assignment is queried
     * @returns int
     * @see class.user.inc
     */
    function get_users($project_id)
    {
      if ($project_id > 0) {
        $query = "SELECT PrAs.User_ID FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
               . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
               . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.Project_ID=$project_id";
        $db = new DB_Management;
        $db->open_db();
        $result = $db->execute_query($query);
        $i=1;
        while ($row = $db->fetch_row($result)) {
          $this->user_id = $row->User_ID;
          $users[$i]=$this;
          $i++;
        }
      $db->close_db();
      return $users;
      } else {
        $error = new Error_Management;
        $error->error_page(160,'');
        die();
      }
    } // end function get_users
    

    /**
     * returns the project role ID of the project assignment
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the project role ID is queried
     * @returns int
     * @see class.project_role.inc
     */
    function get_project_role_id($p_assignment_id)
    {
      if ($p_assignment_id > 0) {

        if (strcmp($this->prid_cache[$p_assignment_id],"")==0) {
          $db = new DB_Management;
          $db->open_db();
          $query = "SELECT PrAs.ProjectRole_ID FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
                 . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
                 . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.P_Assignment_ID= $p_assignment_id";
          $this->prid_cache[$p_assignment_id] =  $db->get_single_value($query);

          $db->close_db();
        }
        $this->project_role_id = $this->prid_cache[$p_assignment_id];
        return $this->project_role_id;
      } else {
        $error = new Error_Management;
        $error->error_page(160,'');
        die();
      }
    } // end function get_project_role_id
    

    /**
     * returns the status of the project assignment, "A"ctiv or "N"ot active
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the status is queried
     * @returns int
     */
    function get_status($p_assignment_id)
    {
      if ($p_assignment_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT PrAs.Status FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
               . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
               . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.P_Assignment_ID= $p_assignment_id";
        $this->status = $db->get_single_value($query);

        $db->close_db();
        return $this->status;
      } else {
        $error = new Error_Management;
        $error->error_page(160,'');
        die();
      }
    } // end function get_status
    
    
    /**
     * returns the description of the project assignment
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the description is queried
     * @returns string
     */
    function get_description($p_assignment_id)
    {
      if ($p_assignment_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query = "SELECT PrAs.Description FROM Project_Assignment AS PrAs, Project AS Proj, User, Project_Role AS PrRo"
               . " WHERE Proj.Project_ID = PrAs.Project_ID AND User.User_ID = PrAs.User_ID AND"
               . " PrRo.ProjectRole_ID = PrAs.ProjectRole_ID AND PrAs.P_Assignment_ID= $p_assignment_id";
        $this->description = $db->get_single_value($query);

        $db->close_db();
        return $this->description;
      } else {
        $error = new Error_Management;
        $error->error_page(160,'');
        die();
      }
    } // end function get_description


    /**
     * checks the validity of the attribute values of a Project_Assignment object and
     * if the validity is ok then returns "" else returns the corresponding check message
     *
     * @param Project_Assignment p_assignment: the Project_Assignment object which has to be checked
     * @param int user_id_log: the user ID of the person which has to be checked
     * @returns string
     */
    function check_params($p_assignment, $user_id_log)
    {
      $db = new DB_Management;
      $db->open_db();

      $check_message = i18n("F&uuml;r den/die Parameter ");
      $check_counter = 0;

      if (($p_assignment->project_id < 1) || ($db->get_count("Project", "Project_ID = $p_assignment->project_id") != 1)) {
        $check_message = $check_message . i18n("Projekt-ID");
        $check_counter++;
      }
      if (($p_assignment->user_id < 1) || ($db->get_count("User", "User_ID = $p_assignment->user_id") != 1)) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Benutzer-ID");
        $check_counter++;
      }
      if (($p_assignment->project_role_id < 1) || ($db->get_count("Project_Role", "ProjectRole_ID = $p_assignment->project_role_id") != 1)) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Benutzerrolle-ID");
        $check_counter++;
      }
      if (!((strcmp($p_assignment->status,"A") == 0) || (strcmp($p_assignment->status,"N") == 0))) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Status");
        $check_counter++;
      }
      
      if ($user_id_log < 1) {
        if ($check_counter>0){
          $check_message = $check_message . ", ";
        }
        $check_message = $check_message . i18n("Bearbeiter-Benutzer-ID");
        $check_counter++;
      }

      $check_message = $check_message . i18n(" wurden keine oder falsche Werte eingegeben! &Uuml;berpr&uuml;fen Sie ihre Eingabe nochmal!");

      if ($check_counter == 0) {
        $check_message="";
      }
      return $check_message;
    } // end function check_params
    
    
    /**
     * sets the project ID of a project assignment and returns "1" on success or "0" on error
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the project ID is set
     * @param int project_id: the project ID which is set
     * @returns int
     * @see class.project.inc
     */
    function set_project_id($p_assignment_id, $project_id)
    {
      if (($p_assignment_id > 0) && ($project_id > 0)){
        $db = new DB_Management;
        $db->open_db();
        if ($db->get_count("Project", "Project_ID=$project_id") == 1){
          $query ="UPDATE Project_Assignment SET Project_ID=$project_id WHERE P_Assignment_ID=$p_assignment_id";
          $update_result=$db->execute_query($query);
          $db->close_db();
          $this->project_id = $project_id;
          $this->pid_cache[$p_assignment_id] = $project_id;
          return $update_result;
        } else {
          $db->close_db();
          $error = new Error_Management;
          $error->error_page(20,'');
          die();
        }
      } else {
        echo "paline736";
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_project_id
    
    
    /**
     * sets the user ID of a project assignment and returns "1" on success or "0" on error
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the user ID is set
     * @param int user_id: the user ID which is set
     * @returns int
     * @see class.user.inc
     */
    function set_user_id($p_assignment_id, $user_id)
    {
      if (($p_assignment_id > 0) && ($user_id > 0)){
        $db = new DB_Management;
        $db->open_db();
        if ($db->get_count("User", "User_ID=$user_id") == 1){
          $query ="UPDATE Project_Assignment SET User_ID=$user_id WHERE P_Assignment_ID=$p_assignment_id";
          $update_result=$db->execute_query($query);
          $db->close_db();
          $this->user_id = $user_id;
          return $update_result;
        } else {
          $db->close_db();
          $error = new Error_Management;
          $error->error_page(30,'');
          die();
        }
      } else {
        echo "paline770";
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_user_id
    
    
    /**
     * sets the project role ID of a project assignment and returns "1" on success or "0" on error
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the project role ID is set
     * @param int project_role_id: the project role ID which is set
     * @returns int
     * @see class.project_role.inc
     */
    function set_project_role_id($p_assignment_id, $project_role_id)
    {
      if (($p_assignment_id > 0) && ($project_role_id > 0)){
        $db = new DB_Management;
        $db->open_db();
        if ($db->get_count("Project_Role", "ProjectRole_ID=$project_role_id") == 1){
          $this->prid_cache[$p_assignment_id] = $project_role_id;
          $query ="UPDATE Project_Assignment SET ProjectRole_ID=$project_role_id WHERE P_Assignment_ID=$p_assignment_id";
          $update_result=$db->execute_query($query);
          $db->close_db();
          $this->project_role_id = $project_role_id;
          return $update_result;
        } else {
          $db->close_db();
          $error = new Error_Management;
          $error->error_page(110,'');
          die();
        }
       } else {
        echo "paline804";
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_project_role_id
    

    /**
     * sets the status of a project assignment and returns "1" on success or "0" on error
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the status is set
     * @param string status: the status which is set
     * @param int user_id_log: the user ID of the user who sets the new status entry
     * @returns int
     */
    function set_status($p_assignment_id, $status, $user_id_log)
    {
      if (($p_assignment_id > 0) && (strcmp($status,"") != 0)){
        $db = new DB_Management;
        $db->open_db();
        if ($db->get_count("User", "User_ID=$user_id_log") == 1){
          $query ="UPDATE Project_Assignment SET Status='$status', User_ID_Log=$user_id_log WHERE P_Assignment_ID=$p_assignment_id";
          $update_result=$db->execute_query($query);
          $db->close_db();
          $this->status = $status;
          return $update_result;
        } else {
          $db->close_db();
          $error = new Error_Management;
          $error->error_page(30,'');
          die();
        }
      } else {
        echo "paline838\n";
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_status
    
    
    /**
     * sets the description of a project assignment and returns "1" on success or "0" on error
     *
     * @param int p_assignment_id: project assignment ID of the project assignment from which one the description is set
     * @param string status: the description which is set
     * @returns int
     */
    function set_description($p_assignment_id, $description)
    {
      if ($p_assignment_id > 0) {
        $db = new DB_Management;
        $db->open_db();

        $query ="UPDATE Project_Assignment SET Description='".$db->encode_string($description)."' WHERE P_Assignment_ID=$p_assignment_id";
        $update_result=$db->execute_query($query);
        $db->close_db();
        $this->description = $description;
        return $update_result;
      } else {
        echo "paline865\n";
        $error = new Error_Management;
        $error->error_page(2,'');
        die();
      }
    } // end function set_description
    
    function setAssignments($userId, $projectRoleId, $projectIdList) {
    	$db = new DB_Management;
    	$db->open_db();
    	$where = "WHERE User_ID=$userId AND ProjectRole_ID=$projectRoleId";
    	$query = "SELECT Project_ID, Status FROM Project_Assignment ".$where;
    	$result = $db->execute_query($query);
    	$pa['A'] = array();
    	$pa['N'] = array();
		while ($row = $db->fetch_row($result)) {
     		$pa[$row->Status][] = $row->Project_ID;
		}
    	$query = "SELECT DISTINCT Project_ID FROM Performance AS P, Project_Assignment AS PA "
    					.$where." AND P.P_Assignment_ID=PA.P_Assignment_ID";
    	$result = $db->execute_query($query);
    	$perf = array();
		while ($row = $db->fetch_row($result)) {
			$perf[] = $row->Project_ID;
		}		
		foreach(array_diff($projectIdList, $pa['A']) as $id) {
			if(in_array($id, $pa['N'])) {
				$query = "UPDATE Project_Assignment SET Status='A' "
								.$where." AND Project_ID=$id";
			}
			else {
				$query = "INSERT INTO Project_Assignment (Project_ID, User_ID, ProjectRole_ID, Status) "
								."VALUES ($id, $userId, $projectRoleId, 'A')";
			}
			$db->execute_query($query);
		}
		foreach(array_diff($pa['A'], $projectIdList) as $id) {
			if(in_array($id, $perf)) {
				$query = "UPDATE Project_Assignment SET Status='N' "
								.$where." AND Project_ID=$id";
			}
			else {
				$query = "DELETE FROM Project_Assignment "
								.$where." AND Project_ID=$id";
			}
			$db->execute_query($query);
		}
		$db->close_db();
     }
    	
 
  } // end class Project_Assignment
?>
